@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: ddev.c,v 1.10 2000/12/04 14:19:38 abe Exp $";
+static char *rcsid = "$Id: ddev.c,v 1.14 2005/08/08 19:46:38 abe Exp $";
 #endif
 
 
@@ -43,7 +43,8 @@ static char *rcsid = "$Id: ddev.c,v 1.10 2000/12/04 14:19:38 abe Exp $";
  * Local definitions
  */
 
-#define	LIKE_NODE_TTL	"like device special "
+#define	LIKE_BLK_SPEC	"like block special"
+#define	LIKE_CHR_SPEC	"like character special"
 
 
 /*
@@ -109,14 +110,15 @@ getchan(p)
 
 
 /*
- * printchdevname() - print character device name
+ * printdevname() - print device name
  */
 
 int
-printchdevname(dev, rdev, f)
+printdevname(dev, rdev, f, nty)
 	dev_t *dev;			/* device */
 	dev_t *rdev;			/* raw device */
 	int f;				/* 1 = follow with '\n' */
+	int nty;			/* node type: N_BLK or N_CHR */
 {
 	struct l_dev *dp;
 /*
@@ -139,16 +141,17 @@ printchdevname(dev, rdev, f)
 	/*
 	 * A raw device match was found.  Record it as a name column addition.
 	 */
-	    char *cp;
+	    char *cp, *ttl;
 	    int len;
 
-	    len = (int)(1 + strlen(LIKE_NODE_TTL) + strlen(dp->name) + 1);
+	    ttl = (nty == N_BLK) ? LIKE_BLK_SPEC : LIKE_CHR_SPEC;
+	    len = (int)(1 + strlen(ttl) + 1 + strlen(dp->name) + 1);
 	    if (!(cp = (char *)malloc((MALLOC_S)(len + 1)))) {
-		(void) fprintf(stderr, "%s: no nma space for: (%s%s)\n",
-		    Pn, LIKE_NODE_TTL, dp->name);
+		(void) fprintf(stderr, "%s: no nma space for: (%s %s)\n",
+		    Pn, ttl, dp->name);
 		Exit(1);
 	    }
-	    (void) snpf(cp, len + 1, "(%s%s)", LIKE_NODE_TTL, dp->name);
+	    (void) snpf(cp, len + 1, "(%s %s)", ttl, dp->name);
 	    (void) add_nma(cp, len);
 	    (void) free((MALLOC_P *)cp);
 	    return(0);
@@ -210,7 +213,7 @@ readdev(skip)
  */
 	if (stat("/dev/clone", &sb) == 0) {
 	    cd = sb.st_rdev;
-	    CloneMaj = major(cd);
+	    CloneMaj = GET_MAJ_DEV(cd);
 	}
 #endif	/* AIXV >=4140 */
 
@@ -251,7 +254,7 @@ readdev(skip)
 	 * Scan the directory.
 	 */
 	    for (dp = readdir(dfp); dp; dp = readdir(dfp)) {
-		if (dp->d_ino == 0 || dp->d_name[0] == '.')
+		if (!dp->d_ino || (dp->d_name[0] == '.'))
 		    continue;
 	    /*
 	     * Form the full path name and get its status.
@@ -317,7 +320,7 @@ readdev(skip)
 			}
 		    }
 		    Devtp[i].rdev = sb.st_rdev;
-		    Devtp[i].inode = sb.st_ino;
+		    Devtp[i].inode = (INODETYPE)sb.st_ino;
 		    if (!(Devtp[i].name = mkstrcpy(fp, (MALLOC_S *)NULL))) {
 			(void) fprintf(stderr, "%s: no space for: ", Pn);
 			safestrprt(fp, stderr, 1);
@@ -330,7 +333,7 @@ readdev(skip)
 		/*
 		 * Save information on AIX 4.1.4 and above clone devices.
 		 */
-		    if (CloneMaj >= 0 && CloneMaj == major(sb.st_rdev)) {
+		    if (CloneMaj >= 0 && CloneMaj == GET_MAJ_DEV(sb.st_rdev)) {
 			if (!(c = (struct clone *)malloc(
 				  (MALLOC_S)sizeof(struct clone))))
 			{
@@ -345,13 +348,13 @@ readdev(skip)
 			    safestrprt(fp, stderr, 1);
 			    exit(1);
 			}
-			c->cd.inode = (unsigned long)sb.st_ino;
+			c->cd.inode = (INODETYPE)sb.st_ino;
 			c->cd.rdev = sb.st_rdev;
 			c->cd.v = 0;
 			c->next = Clone;
 			Clone = c;
 			if (ClonePtc < 0 && strcmp(path, "/dev/ptc") == 0)
-			    ClonePtc = minor(sb.st_rdev);
+			    ClonePtc = GET_MIN_DEV(sb.st_rdev);
 		    }
 #endif	/* AIXV >=4140 */
 
@@ -379,7 +382,7 @@ readdev(skip)
 			}
 		    }
 		    BDevtp[j].rdev = sb.st_rdev;
-		    BDevtp[j].inode = sb.st_ino;
+		    BDevtp[j].inode = (INODETYPE)sb.st_ino;
 		    BDevtp[j].name = fp;
 		    fp = (char *)NULL;
 		    BDevtp[j].v = 0;
@@ -627,14 +630,14 @@ bad_cached_clone:
 		    }
 		    return(1);
 		}
-		CloneMaj = major(c->cd.rdev);
+		CloneMaj = GET_MAJ_DEV(c->cd.rdev);
 	    /*
 	     * Enter the clone inode number.
 	     */
-		for (c->cd.inode = 0l; *cp != ' '; cp++) {
+		for (c->cd.inode = (INODETYPE)0; *cp != ' '; cp++) {
 		    if (*cp < '0' || *cp > '9')
 			goto bad_cached_clone;
-		    c->cd.inode = (c->cd.inode * 10l) + (long)(*cp - '0');
+		    c->cd.inode = (INODETYPE)((c->cd.inode * 10) + (*cp - '0'));
 		}
 	    /*
 	     * Enter the clone path name.
@@ -658,7 +661,7 @@ bad_cached_clone:
 		c->next = Clone;
 		Clone = c;
 		if (ClonePtc < 0 && strcmp(c->cd.name, "/dev/ptc") == 0)
-		    ClonePtc = minor(c->cd.rdev);
+		    ClonePtc = GET_MIN_DEV(c->cd.rdev);
 	    }
 	    return(0);
 	} else if (m == 2) {
@@ -715,7 +718,7 @@ vfy_dev(dp)
 #endif	/* defined(USE_STAT) */
 
 	||  dp->rdev != sb.st_rdev
-	||  dp->inode != sb.st_ino) {
+	||  dp->inode != (INODETYPE)sb.st_ino) {
 	   (void) rereaddev();
 	   return(0);
 	}
