@@ -32,31 +32,38 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.19 2001/07/18 14:01:42 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.26 2008/10/21 16:14:18 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
-
 _PROTOTYPE(static void get_kernel_access,(void));
 
-#if	AIXA<1
+#if	AIXA<2
 _PROTOTYPE(static struct le *getle,(KA_T a, KA_T sid, char **err));
-#endif	/* AIXA<1 */
+#endif	/* AIXA<2 */
 
 #if	AIXV>=4110
-_PROTOTYPE(static void getlenm,(struct le *le));
+_PROTOTYPE(static void getlenm,(struct le *le, KA_T sid));
 #endif	/* AIXV>=4110 */
 
 _PROTOTYPE(static int kreadx,(KA_T addr, char *buf, int len, KA_T sid));
 
-#if	AIXA<1
+#if	AIXA<2
 _PROTOTYPE(static void process_text,(KA_T sid));
-#else	/* AIXA>=1 */
+#else	/* AIXA>=2 */
 _PROTOTYPE(static void getsoinfo,(void));
 _PROTOTYPE(static void process_text,(pid_t pid));
-#endif	/* AIXA<1 */
+#endif	/* AIXA<2 */
+
+#if	defined(SIGDANGER)
+# if	defined(HASINTSIGNAL)
+_PROTOTYPE(static int lowpgsp,(int sig));
+# else	/* !defined(HASINTSIGNAL) */
+_PROTOTYPE(static void lowpgsp,(int sig));
+# endif	/* defined(HASINTSIGNAL) */
+#endif	/* defined(SIGDANGER) */
 
 
 /*
@@ -76,6 +83,7 @@ _PROTOTYPE(static void process_text,(pid_t pid));
 #define	GETPROCS	getprocs64
 #define	PROCINFO	procentry64
 
+#  if	AIXA>1
 /*
  * AIX 5 and greater ia64 loader definitions
  */
@@ -84,24 +92,41 @@ _PROTOTYPE(static void process_text,(pid_t pid));
 
 #define	SOHASHBUCKS	128		/* SoHash[] bucket count
 					 * MUST BE A POWER OF 2!!! */
-#define	SOHASH(d, n)	((((int)(((minor64(d) & 0x7fffffff) * SOHASHBUCKS) \
+#define	SOHASH(d, n)	((((int)(((GET_MIN_DEV(d) & 0x7fffffff) * SOHASHBUCKS) \
 			         + n) * 31415) >> 7) & (SOHASHBUCKS - 1))
 
 typedef struct so_hash {
 	dev_t dev;			/* device (st_dev) */
 	int nlink;			/* link count (st_nlink) */
 	char *nm;			/* name (mi_name) */
-	unsigned long node;		/* node number (st_ino) */
+	INODETYPE node;			/* node number (st_ino) */
 	struct so_hash *next;		/* next entry in hash bucket */
 	SZOFFTYPE sz;			/* size (st_size) */
 } so_hash_t;
 
 so_hash_t **SoHash = (so_hash_t **)NULL;
+#  endif	/* AIXA>1 */
 # endif	/* AIXA<1 */
 #endif	/* AIXV<4300 */
 
 #define	PROCSIZE	sizeof(struct PROCINFO)
 
+/*
+ * Create the FDSINFOSIZE definition for allocating FDSINFO space.  (This
+ * isn't as straightforward as it might seem, because someone made a bad
+ * decision to change the struct fdsinfo* family at AIX 5.2.)
+ */
+
+#define	FDSINFOSIZE	sizeof(struct FDSINFO)	/* (If we're lucky.) */
+
+#if	defined(OPEN_SHRT_MAX)
+# if	OPEN_SHRT_MAX<OPEN_MAX
+#undef	FDSINFOSIZE				/* (We weren't lucky.) */
+#define	FDSELEMSIZE	(sizeof(struct FDSINFO)/OPEN_SHRT_MAX)
+#define	FDSINFOSIZE	(OPEN_MAX * FDSELEMSIZE)
+# endif	/* OPEN_SHRT_MAX<OPEN_MAX */
+#endif	/* defined(OPEN_SHRT_MAX) */
+
 
 #if	AIXV>=4110
 /*
@@ -110,10 +135,16 @@ so_hash_t **SoHash = (so_hash_t **)NULL;
 
 #define	LIBNMLN		40			/* maximum library table name
 						 * length */
+
 #define	LIBMASK		0xf0000000		/* library table mask */
 #define	LIBNMCOMP	0xd0000000		/* library table name has
 						 * multiple components */
+# if	AIXA<1
 #define	RDXMASK		0x0fffffff		/* kreadx() address mask */
+# else	/* AIXA>=1 */
+#define	RDXMASK		0x0fffffffffffffff	/* kreadx() address mask */
+#define	URDXMASK	0x0fffffff00000000	/* upper part of RDXMASK */
+# endif	/* AIXA<1 */
 #endif	/* AIXV>=4110 */
 
 
@@ -140,16 +171,16 @@ struct le {				/* loader entry */
 	char *nm;			/* name */
 # endif	/* AIXV>=4110 */
 #else	/* AIXV>=4300 */
-# if	AIXA<1
+# if	AIXA<2
 	uint flags;
 	struct file *fp;		/* file table entry pointer */
 	char *nm;			/* name */
-# else	/* AIXA>=1 */
+# else	/* AIXA>=2 */
 	KA_T d1[2];
 	KA_T nm;			/* name */
 	KA_T d2[10];
 	struct file *fp;		/* file table entry pointer */
-# endif	/* AIXA<1 */
+# endif	/* AIXA<2 */
 #endif	/* AIXV<4300 */
 
 };
@@ -161,13 +192,13 @@ struct le {				/* loader entry */
  */
 struct la {			/* loader anchor */
 
-# if	AIXA<1
+# if	AIXA<2
     struct le *list;
     struct le *exec;
-# else	/* AIXA>=1 */
+# else	/* AIXA>=2 */
     KA_T exec;
     KA_T list;
-# endif	/* AIXA<1 */
+# endif	/* AIXA<2 */
 };
 #endif	/* AIXV>=4300 */
 
@@ -182,10 +213,10 @@ static struct PROCINFO *P = (struct PROCINFO *)NULL;
 static struct user *Up;			/* user structure */
 
 #if	AIXV>=4110
-# if	AIXA<1
+# if	AIXA<2
 static KA_T Soff;			/* shared library VM offset */
 int Soff_stat = 0;			/* Soff-available status */
-# endif	/* AIXA<1 */
+# endif	/* AIXA<2 */
 static KA_T Uo;				/* user area VM offset */
 #endif	/* AIXV>=4110 */
 
@@ -203,6 +234,11 @@ ckkv(d, er, ev, ea)
 {
 
 #if	defined(HASKERNIDCK)
+# if	AIXV<5000
+
+/*
+ * Use oslevel below AIX 5.
+ */
 	int br, p[2], pid;
 	char buf[128], *cp;
 	struct stat sb;
@@ -219,10 +255,10 @@ ckkv(d, er, ev, ea)
 	
 	if (stat(OSLEVELPATH, &sb)) {
 
-# if	AIXV<4100
+#  if	AIXV<4100
 	    if (errno == ENOENT)
 		return;
-# endif	/* AIXV<4100 */
+#  endif	/* AIXV<4100 */
 
 	    (void) fprintf(stderr, "%s: can't execute %s: %s\n",
 		Pn, OSLEVELPATH, strerror(errno));
@@ -270,6 +306,19 @@ ckkv(d, er, ev, ea)
 		*cp = '\0';
 	} else
 	    (void) snpf(buf, sizeof(buf), "UNKNOWN");
+# else	/* AIXV>=5000 */
+
+/*
+ * Use uname() for AIX 5 and above.
+ */
+	char buf[64];
+	struct utsname u;
+
+	(void) memset((void *)&u, 0, sizeof(u));
+	(void) uname(&u);
+	(void) snpf(buf, sizeof(buf) - 1, "%s.%s.0.0", u.version, u.release);
+	buf[sizeof(buf) - 1] = '\0';
+# endif	/* AIXV<5000 */
 	if (!ev || strcmp(buf, ev))
 	    (void) fprintf(stderr,
 		"%s: WARNING: compiled for %s version %s; this is %s.\n",
@@ -286,6 +335,16 @@ ckkv(d, er, ev, ea)
 void
 gather_proc_info()
 {
+	short cckreg;			/* conditional status of regular file
+					 * checking:
+					 *     0 = unconditionally check
+					 *     1 = conditionally check */
+	short ckscko;			/* socket file only checking status:
+					 *     0 = none
+					 *     1 = check only socket files,
+					 *	   including TCP and UDP
+					 *	   streams with eXPORT data,
+					 *	   where supported */
 	KA_T cdir, fp, pdir, rdir;
 	char *cmd;
 	int hl, i, nf, np;
@@ -296,7 +355,17 @@ gather_proc_info()
 #if	AIXV>=4300
 	static struct FDSINFO *fds = (struct FDSINFO *)NULL;
 	MALLOC_S msz;
+# if	AIXA==1
+	pid32_t pid;		/* Since we're operating with types defined
+				 * under _KERNEL (see machine.), but
+				 * getprocs64() expects application types
+				 * (where pid_t is 32 bits), the pid variable
+				 * must be cast in an application-compatible
+				 * manner.
+				 */
+# else	/* AIXA!=1 */
 	pid_t pid;
+# endif	/* AIXA==1 */
 # if	AIXV==4330
 	static int trx = 0;
 	unsigned int mxof;
@@ -304,6 +373,55 @@ gather_proc_info()
 # endif	/* AIXV==4330 */
 #endif	/* AIXV>=4300 */
 
+/*
+ * Define socket and regular file conditional processing flags.
+ *
+ * If only socket files have been selected, or socket files have been
+ * selected, ANDed with other selection options, enable the skipping of
+ * regular files.
+ *
+ * If socket files and some process options have been selected, enable
+ * conditional skipping of regular file; i.e., regular files will be skipped
+ * unless they belong to a process selected by one of the specified options.
+ */
+	if (Selflags & SELNW) {
+
+	/*
+	 * Some network files selection options have been specified.
+	 */
+	    if (Fand || !(Selflags & ~SELNW)) {
+
+	    /*
+	     * Selection ANDing or only network file options have been
+	     * specified, so set unconditional skipping of regular files
+	     * and socket file only checking.
+	     */
+		cckreg = 0;
+		ckscko = 1;
+	    } else {
+
+	    /*
+	     * If ORed file selection options have been specified, or no
+	     * ORed process selection options have been specified, enable
+	     * unconditional file checking and clear socket file only
+	     * checking.
+	     *
+	     * If only ORed process selection options have been specified,
+	     * enable conditional file skipping and socket file only checking.
+	     */
+		if ((Selflags & SELFILE) || !(Selflags & SELPROC))
+		    cckreg = ckscko = 0;
+		else
+		    cckreg = ckscko = 1;
+	    }
+	} else {
+
+	/*
+	 * No network file selection options were specified.  Enable
+	 * unconditional file checking and clear socket file only checking.
+	 */
+	    cckreg = ckscko = 0;
+	}
 /*
  * Read the process table.
  */
@@ -412,6 +530,16 @@ gather_proc_info()
 	    nf = Up->u_maxofile;
 	    if (is_cmd_excl(cmd, &pss, &sf))
 		continue;
+	    if (cckreg) {
+
+	    /*
+	     * If conditional checking of regular files is enabled, enable
+	     * socket file only checking, based on the process' selection
+	     * status.
+	     */
+		ckscko = (sf & SELPROC) ? 0 : 1;
+	    }
+	    
 #else	/* AIXV>=4300 */
 	/*
 	 * For AIX 4.3 and above, skip processes excluded by the procsinfo
@@ -422,9 +550,17 @@ gather_proc_info()
 	 */
 	    if (is_cmd_excl(p->pi_comm, &pss, &sf))
 		continue;
+	    if (cckreg) {
+
+	    /*
+	     * If conditional checking of regular files is enabled, enable
+	     * socket file only checking, based on the process' selection
+	     * status.
+	     */
+		ckscko = (sf & SELPROC) ? 0 : 1;
+	    }
 	    if (!fds) {
-		if (!(fds = (struct FDSINFO *)malloc((MALLOC_S)
-					      sizeof(struct FDSINFO))))
+		if (!(fds = (struct FDSINFO *)malloc((MALLOC_S)FDSINFOSIZE)))
 		{
 		    (void) fprintf(stderr,
 			"%s: can't allocate fdsinfo struct for PID %d\n",
@@ -433,8 +569,8 @@ gather_proc_info()
 		}
 	    }
 	    pid = p->p_pid;
-	    if (GETPROCS((struct PROCINFO *)NULL, PROCSIZE, fds,
-			 sizeof(struct FDSINFO), &pid, 1)
+	    if (GETPROCS((struct PROCINFO *)NULL, PROCSIZE, fds, FDSINFOSIZE,
+			  &pid, 1)
 	    != 1)
 		continue;
 	    hl = 0;
@@ -526,7 +662,7 @@ gather_proc_info()
 	/*
 	 * Save current working directory information.
 	 */
-	    if (cdir) {
+	    if (!ckscko && cdir) {
 		alloc_lfile(CWD, -1);
 		process_node(cdir);
 		if (Lf->sf)
@@ -535,7 +671,7 @@ gather_proc_info()
 	/*
 	 * Save root directory information.
 	 */
-	    if (rdir) {
+	    if (!ckscko && rdir) {
 		alloc_lfile(RTD, -1);
 		process_node(rdir);
 		if (Lf->sf)
@@ -546,7 +682,7 @@ gather_proc_info()
 	/*
 	 * Save parent directory information.
 	 */
-	    if (pdir) {
+	    if (!ckscko && pdir) {
 		alloc_lfile("  pd", -1);
 		process_node(pdir);
 		if (Lf->sf)
@@ -557,13 +693,39 @@ gather_proc_info()
 	/*
 	 * Save information on text files.
 	 */
-	    if (hl) {
+	    if (!ckscko && hl) {
 
-#if	AIXA<1
+#if	AIXA<2
+# if	AIXA<1
 		process_text((KA_T)p->pi_adspace);
-#else	/* AIXA>=1 */
+# else	/* AIXA==1 */
+		{
+		    int ck = 1;
+		    KA_T sid = (KA_T)p->pi_adspace;
+
+		    if ((Up->U_loader[0] & URDXMASK)
+		    ||  (Up->U_loader[1] & URDXMASK))
+		    {
+
+		    /*
+		     * If the upper part of either loader map address is
+		     * non-zero and this is not the lsof process, skip the
+		     * processing of text files.  If this is the lsof process,
+		     * set the segment address to zero, forcing text file
+		     * information to come from kmem rather than mem.
+		     */
+			if (Mypid == p->p_pid)
+			    sid = (KA_T)0;
+			else
+			    ck = 0;
+		    }
+		    if (ck)
+			process_text(sid);
+		}
+# endif	/* AIXA<1 */
+#else	/* AIXA>=2 */
 		process_text(p->p_pid);
-#endif	/* AIXA<1 */
+#endif	/* AIXA<2 */
 
 	    }
 	/*
@@ -613,6 +775,39 @@ static void
 get_kernel_access()
 {
 	int oe = 0;
+
+#if	defined(AIX_KERNBITS)
+	int kb;
+	char *kbb, *kbr;
+/*
+ * Check the kernel bit size against the size for which this executable was
+ * configured.
+ */
+	if (__KERNEL_32()) {
+	    kb = 32;
+	    kbr = "32";
+	} else if (__KERNEL_64()) {
+	    kb = 64;
+	    kbr = "64";
+	} else {
+	    kb = 0;
+	    kbr = "unknown";
+	}
+	if ((AIX_KERNBITS == 0) || !kb || (kb != AIX_KERNBITS)) {
+	    if (AIX_KERNBITS == 32)
+		kbb = "32";
+	    else if (AIX_KERNBITS == 64)
+		kbb = "64";
+	    else
+		kbb = "unknown";
+	    (void) fprintf(stderr,
+		"%s: FATAL: compiled for a kernel of %s bit size.\n", Pn, kbb);
+	    (void) fprintf(stderr,
+		"      The bit size of this kernel is %s.\n", kbr);
+	    Exit(1);
+	}
+#endif	/* defined(AIX_KERNBITS) */
+
 /*
  * Access /dev/mem.
  */
@@ -677,7 +872,7 @@ get_kernel_access()
 	if (Fxopt) {
 	    struct ublock *ub;
 
-# if	AIXA<1
+# if	AIXA<2
 	    struct nlist ll[] = {
 		{ "library_anchor"	},
 
@@ -696,10 +891,10 @@ get_kernel_access()
 			sizeof(Soff), (KA_T)0)
 	    == 0)
 		Soff_stat++;
-# endif	/* AIXA<1 */
+# endif	/* AIXA<2 */
 
 	    ub = (struct ublock *)Nl[X_UADDR].n_value;
-	    Uo = (KA_T)((int)&ub->ub_user & RDXMASK);
+	    Uo = (KA_T)((KA_T)&ub->ub_user & RDXMASK);
 	}
 #endif	/* AIXV>=4110 */
 
@@ -707,10 +902,18 @@ get_kernel_access()
  * Check the kernel version number.
  */
 	(void) ckkv("AIX", (char *)NULL, LSOF_VSTR, (char *)NULL);
+
+#if	defined(SIGDANGER)
+/*
+ * If SIGDANGER is defined, enable its handler.
+ */
+	(void) signal(SIGDANGER, lowpgsp);
+#endif	/* defined(SIGDANGER) */
+
 }
 
 
-#if	AIXA<1
+#if	AIXA<2
 /*
  * getle() - get loader entry structure
  */
@@ -738,16 +941,24 @@ getle(a, sid, err)
 	}
 
 #if	AIXV>=4110
-	if (!kreadx((KA_T)(a & RDXMASK), (char *)&le, sizeof(le), (KA_T)sid))
+	if (!sid) {
+	    if (!kread(a, (char *)&le, sizeof(le)))
+		return(&le);
+	} else {
+	    if (!kreadx((KA_T)(a & RDXMASK),(char *)&le,sizeof(le),(KA_T)sid))
+		return(&le);
+	}
 #else	/* AIXV<4110 */
 	if (!kreadx((KA_T)a, (char *)&le, sizeof(le), (KA_T)sid))
+	    return(&le);
 #endif	/* AIXV>=4110 */
 
-	    return(&le);
+getle_err:
+
 	*err = "can't readx()";
 	return((struct le *)NULL);
 }
-#endif	/* AIXA<1 */
+#endif	/* AIXA<2 */
 
 
 #if	AIXV>=4110
@@ -756,8 +967,9 @@ getle(a, sid, err)
  */
 
 static void
-getlenm(le)
-	struct le *le;
+getlenm(le, sid)
+	struct le *le;			/* loader entry structure */
+	KA_T sid;			/* segment ID */
 {
 	char buf[LIBNMLN];
 	int i;
@@ -766,31 +978,36 @@ getlenm(le)
 	if ((le->ft & LIBMASK) != LIBNMCOMP)
 	    return;
 #else	/* AIXV>=4300 */
-# if	AIXA<1
-	if (Soff_stat
-	&&  !kreadx((KA_T)le->nm & (KA_T)RDXMASK, buf, LIBNMLN, (KA_T)Soff)) {
-	    buf[LIBNMLN - 1] = '\0';
-	    i = strlen(buf);
-	    if (i < (LIBNMLN - 3) && buf[i+1])
-		enter_nm(&buf[i+1]);
-	    else if (buf[0])
-		enter_nm(buf);
+# if	AIXA<2
+	if (!sid) {
+	     if (kread((KA_T)le->nm, buf, LIBNMLN))
+		return;
+	} else {
+	    if (!Soff_stat || !le->nm
+	    ||  kreadx((KA_T)le->nm & (KA_T)RDXMASK, buf, LIBNMLN, (KA_T)Soff))
+		return;
 	}
-# else	/* AIXA>=1 */
+	buf[LIBNMLN - 1] = '\0';
+	i = strlen(buf);
+	if (i < (LIBNMLN - 3) && buf[i+1])
+	    enter_nm(&buf[i+1]);
+	else if (buf[0])
+	    enter_nm(buf);
+# else	/* AIXA>=2 */
 	if (!le->nm || kread(le->nm, buf, sizeof(buf)))
 	    return;
 	buf[LIBNMLN - 1] = '\0';
 	if (!strlen(buf))
 	    return;
 	enter_nm(buf);
-# endif	/* AIXA<1 */
+# endif	/* AIXA<2 */
 #endif	/* AIXV<4300 */
 
 }
 #endif	/* AIXV>=4110 */
 
 
-#if	AIXA>=1
+#if	AIXA>1
 /*
  * getsoinfo() - get *.so information for ia64 AIX >= 5
  */
@@ -882,12 +1099,12 @@ getsoinfo()
 	    sp->nlink = (int)sb.st_nlink;
 	    sp->nm = ln;
 	    ln = (char *)NULL;
-	    sp->node = (unsigned long)sb.st_ino;
+	    sp->node = (INODETYPE)sb.st_ino;
 	    sp->sz = (SZOFFTYPE)sb.st_size;
 	/*
 	 * Link the structure to the appropriate hash bucket.
 	 */
-	    h = SOHASH(sb.st_dev, sb.st_ino);
+	    h = SOHASH(sb.st_dev, (INODETYPE)sb.st_ino);
 	    if (SoHash[h])
 		sp->next = SoHash[h];
 	    else
@@ -902,7 +1119,7 @@ getsoinfo()
 	if (rn)
 	    (void) free((MALLOC_P *)rn);
 }
-#endif	/* AIXA>=1 */
+#endif	/* AIXA>1 */
 
 
 /*
@@ -914,9 +1131,9 @@ initialize()
 {
 	get_kernel_access();
 
-#if	AIXA>=1
+#if	AIXA>1
 	(void) getsoinfo();
-#endif	/* AIXA>=1 */
+#endif	/* AIXA>1 */
 
 }
 
@@ -970,7 +1187,27 @@ kreadx(addr, buf, len, sid)
 }
 
 
-#if	AIXA<1
+#if	defined(SIGDANGER)
+/*
+ * lowpgsp() - hangle a SIGDANGER signal about low paging space
+*/
+
+# if	defined(HASINTSIGNAL)
+static int
+# else	/* !defined(HASINTSIGNAL) */
+static void
+# endif	/* defined(HASINTSIGNAL) */
+
+lowpgsp(sig)
+	int sig;
+{
+	(void) fprintf(stderr, "%s: FATAL: system paging space is low.\n", Pn);
+	Exit(1);
+}
+#endif	/* defined(SIGDANGER) */
+
+
+#if	AIXA<2
 /*
  * process_text() - process text file information for non-ia64 AIX
  */
@@ -1011,7 +1248,7 @@ process_text(sid)
 
 #if	AIXV>=4110 && AIXV<4300
 			if (!Lf->nm || !Lf->nm[0])
-			    getlenm(le);
+			    getlenm(le, sid);
 #endif	/* AIXV>=4110 && AIXV<4300 */
 
 			link_lfile();
@@ -1087,7 +1324,7 @@ process_text(sid)
 
 #if	AIXV>=4110
 		if (!Lf->nm || !Lf->nm[0])
-		    getlenm(le);
+		    getlenm(le, sid);
 #endif	/* AIXV>=4110 */
 
 		link_lfile();
@@ -1095,7 +1332,7 @@ process_text(sid)
 	    }
 	}
 }
-#else	/* AIXA>=1 */
+#else	/* AIXA>=2 */
 /*
  * process_text() - process text file information for ia64 AIX >= 5
  */
@@ -1116,7 +1353,7 @@ process_text(pid)
 	so_hash_t *sp;
 	size_t sz;
 	dev_t xdev;
-	unsigned long xnode;
+	INODETYPE xnode;
 	int xs = 0;
 /*
  * Display information on the exec'd entry.
@@ -1132,7 +1369,7 @@ process_text(pid)
 	    }
 	    if (Lf->sf) {
 		if (!Lf->nm || !Lf->nm[0])
-		    getlenm(&le);
+		    getlenm(&le, (KA_T)0);
 		link_lfile();
 	    }
 	}
@@ -1173,7 +1410,7 @@ process_text(pid)
 	/*
 	 * Ignore the exec'd and non-regular files.
 	 */
-	    if (xs && (xdev == sb.st_dev) && (xnode == sb.st_ino))
+	    if (xs && (xdev == sb.st_dev) && (xnode == (INODETYPE)sb.st_ino))
 		continue;
 	    if (!S_ISREG(sb.st_mode))
 		continue;
@@ -1184,13 +1421,18 @@ process_text(pid)
 	    alloc_lfile(fd, -1);
 	    Lf->dev_def = Lf->inp_ty = Lf->nlink_def = Lf->sz_def = 1;
 	    Lf->dev = sb.st_dev;
-	    Lf->inode = (unsigned long)sb.st_ino;
+	    Lf->inode = (INODETYPE)sb.st_ino;
 	    (void) snpf(Lf->type, sizeof(Lf->type), "VREG");
 	/*
 	 * Look for a match on device and node numbers in the *.so cache.
 	 */
-	    for (sp = SoHash[SOHASH(sb.st_dev,sb.st_ino)]; sp; sp = sp->next) {
-		if ((sp->dev == sb.st_dev) && (sp->node == sb.st_ino)) {
+	    for (sp = SoHash[SOHASH(sb.st_dev, (INODETYPE)sb.st_ino)];
+		 sp;
+		 sp = sp->next)
+	    {
+		if ((sp->dev == sb.st_dev)
+		&&  (sp->node == (INODETYPE)sb.st_ino))
+		{
 
 		/*
 		 * A match was found; use its name, link count, and size.
@@ -1216,7 +1458,7 @@ process_text(pid)
 	 */
 
 # if	defined(HAS_NFS)
-	    if (Fnfs && (minor64(Lf->dev_def) & SDEV_REMOTE))
+	    if (Fnfs && (GET_MIN_DEV(Lf->dev_def) & SDEV_REMOTE))
 		Lf->sf |= SELNFS;
 # endif	/* defined(HAS_NFS) */
 
@@ -1236,4 +1478,4 @@ process_text(pid)
 	}
 	(void) closedir(dfp);
 }
-#endif	/* AIXA<1 */
+#endif	/* AIXA<2 */
