@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.41 2008/10/21 16:16:06 abe Exp $
+ * $Id: dlsof.h,v 1.47 2015/07/07 20:23:43 abe Exp $
  */
 
 
@@ -66,8 +66,37 @@
 #   endif	/* FREEBSDV<6020 */
 #define	_KERNEL	1
 #  endif	/* FREEBSDV>=5000 */
+
+#  if	defined(HAS_VM_MEMATTR_T)
+/*
+ * The d_mmap2_t function typedef in <sys/conf.h> may need the definition
+ * of vm_memattr_t for a pointer, but that definition is only available
+ * under _KERNEL in <sys/types.h>.  Defining _KERNEL before including
+ * <sys/types.h> causes many compilation problems, so this expediency
+ * (hack) is used when the vm_memattr_t definition is needed.
+ */
+#define	vm_memattr_t	void
+#  endif	/* defined(HAS_VM_MEMATTR_T) */
+
+#  if	defined(NEEDS_BOOLEAN_T)
+/*
+ * In FreeBSD 9 and above the boolean_t typedef is also needed and is also
+ * under _KERNEL in <sys/types.h>.
+ */
+
+#define	boolean_t	int
+#  endif	/* defined(NEEDS_BOOLEAN_T) */
+
 #include <sys/conf.h>
 
+#  if	defined(HAS_VM_MEMATTR_T)
+#undef	vm_memattr_t
+#  endif	/* defined(HAS_VM_MEMATTR_T) */
+
+#  if	defined(NEEDS_BOOLEAN_T)
+#undef	boolean_t
+#  endif	/* defined(NEEDS_BOOLEAN_T) */
+
 #  if	defined(HAS_CONF_MINOR)
 #undef	minor
 #include "fbsd_minor.h"
@@ -90,6 +119,15 @@
 #include <sys/mount.h>
 
 # if	FREEBSDV>=3020
+#  if	defined(__clang__)
+/*
+ * This definition is needed when clang is used, because <sys/mount.h> must
+ * be #include'd when _KERNEL is defined and that causes the getmntinfo()
+ * function prototype to be skipped.
+ */
+int     getmntinfo(struct statfs **, int);
+#  endif	/* defined(__clang__) */
+
 #undef	_KERNEL
 # endif	/* FREEBSDV>=3020 */
 
@@ -158,7 +196,24 @@ struct vop_setextattr_args;
 #include <ufs/ufs/extattr.h>
 #define	psignal	LSOF_psignal
 #define	panicstr bp
+
+#  if	defined(__clang__)
+/*
+ * Two clang work-arounds...
+ */
+#define	KASSERT(exp,msg) do {} while (0)
+#include <arpa/inet.h>
+#  endif	/* defined(__clang__) */
+
 #include <ufs/ufs/ufsmount.h>
+
+#  if	defined(__clang__)
+/*
+ * Undo the clang work-arounds.
+ */
+#undef	KASSERT
+#  endif	/* defined(__clang__) */
+
 #undef	psignal
 #undef	panicstr
 #undef	_KERNEL
@@ -186,9 +241,9 @@ struct vop_setextattr_args;
 #include <nfs/nfsproto.h>
 # endif	/* FREEBSDV<5000 */
 
-# if	defined(HASRPCV2H) || FREEBSDV>=4000
+# if	defined(HASRPCV2H)
 #include <nfs/rpcv2.h>
-# endif	/* defined(HASRPCV2H) || FREEBSDV>=4000 */
+# endif	/* defined(HASRPCV2H) */
 
 # if	FREEBSDV>=5000
 #include <nfsclient/nfs.h>
@@ -285,6 +340,8 @@ struct vop_generic_args;
 #  endif	/* FREEBSDV<5000 */
 # endif	/* FREEBSDV<2000 */
 
+#include <vm/vm.h>
+
 #define	_KERNEL
 #define	KERNEL
 #include <sys/fcntl.h>
@@ -299,10 +356,25 @@ struct vop_generic_args;
  * versions where testing was not possible.
  */
 
+#  if	defined(__clang__)
+/*
+ * This work-around is needed when using clang, because <sys/fcntl.h> must
+ * be #include'd under KERNEL and that causes the open() function prototype
+ * definition to be skipped.
+ */
+int     open(const char *, int, ...);
+#  endif	/* defined(__clang__) */
+
 #define	intrmask_t	int
 #define	log	log_kernel_lsof
+
+# if	!defined(HAS_PAUSE_SBT)
 #define	pause	pause_kernel_lsof
+# endif	/* !defined(HAS_PAUSE_SBT) */
+
+#define	asprintf asprintf_kernel_lsof
 #define	setenv	setenv_kernel_lsof
+#define	vasprintf vasprintf_kernel_lsof
 #define	uintfptr_t	int
 #define	_SYS_LIBKERN_H_
 #include <sys/file.h>
@@ -312,10 +384,16 @@ struct vop_generic_args;
  */
 
 #undef	_SYS_LIBKERN_H_
+#undef	asprintf_kernel_lsof
 #undef	intrmask_t_lsof
 #undef	log_kernel_lsof
+
+# if	!defined(HAS_PAUSE_SBT)
 #undef	pause_kernel_lsof
+# endif	/* !defined(HAS_PAUSE_SBT) */
+
 #undef	setenv_kernel_lsof
+#undef	vasprintf_kernel_lsof
 #undef	uintfptr_t
 #undef	_KERNEL
 #undef	KERNEL
@@ -345,9 +423,9 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
 #include <sys/sx.h>
 #   endif	/* defined(HAS_SYS_SX_H) */
 
-#   if	defined(HAS_SI_PRIV) || defined(HAS_CONF_MINOR)
+#   if	defined(HAS_SI_PRIV) || defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
 #include <fs/devfs/devfs_int.h>
-#   endif	/* defined(SI_PRIV) || defined(HAS_CONF_MINOR) */
+#   endif	/* defined(SI_PRIV) || defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV) */
 
 #include <fs/devfs/devfs.h>
 #undef	_KERNEL
@@ -355,8 +433,6 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
 #include <sys/lockf.h>
 # endif	/* FREEBSDV<2000 */
 
-#include <vm/vm.h>
-
 #  if   FREEBSDV>=2020
 #   if	FREEBSDV>=4090
 #define	_KERNEL
@@ -378,27 +454,27 @@ struct vop_advlock_args { int dummy; };	/* to pacify lf_advlock() prototype */
  *  This work-around was supplied by John Polstra <jdp@polstra.com>.
  */
 
-#if	defined(MAP_ENTRY_IS_SUB_MAP) && !defined(MAP_ENTRY_IS_A_MAP)
+# if	defined(MAP_ENTRY_IS_SUB_MAP) && !defined(MAP_ENTRY_IS_A_MAP)
 #define MAP_ENTRY_IS_A_MAP	0
-#endif	/* defined(MAP_ENTRY_IS_SUB_MAP) && !defined(MAP_ENTRY_IS_A_MAP) */
+# endif	/* defined(MAP_ENTRY_IS_SUB_MAP) && !defined(MAP_ENTRY_IS_A_MAP) */
 
 #include <vm/vm_object.h>
 #include <vm/vm_pager.h>
 
-#  if   FREEBSDV>=2020
+# if   FREEBSDV>=2020
 #undef	B_NEEDCOMMIT
 
-#   if	FREEBSDV>=5000
+#  if	FREEBSDV>=5000
 #include <sys/bio.h>
-#   endif	/* FREEBSDV>=5000 */
+#  endif	/* FREEBSDV>=5000 */
 
 #include <sys/buf.h>
 #include <sys/user.h>
 
-#   if	FREEBSDV<5000
+#  if	FREEBSDV<5000
 #include <ufs/mfs/mfsnode.h>
-#   endif	/* FREEBSDV<5000 */
-#  endif        /* FREEBSDV>=2020 */
+#  endif	/* FREEBSDV<5000 */
+# endif        /* FREEBSDV>=2020 */
 
 #include <string.h>
 
@@ -592,9 +668,9 @@ struct	namecache {
 #  endif	/* DEFINED(HASNCVPID) */
 # endif  /* defined(HASNCACHE) */
 
-#if	FREEBSDV>=5000
+# if	FREEBSDV>=5000
 #define	VNODE_VFLAG	v_iflag
 #define	NCACHE_VROOT	VV_ROOT
-#endif	/* FREEBSDV>=5000 */
+# endif	/* FREEBSDV>=5000 */
 
 #endif	/* defined(FREEBSD_LSOF_H) */
