@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.38 2008/05/09 12:53:13 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.44 2015/07/07 20:23:43 abe Exp $";
 #endif
 
 
@@ -51,6 +51,13 @@ static char *rcsid = "$Id: dnode.c,v 1.38 2008/05/09 12:53:13 abe Exp $";
 _PROTOTYPE(static int lkup_dev_tty,(dev_t *dr, INODETYPE *ir));
 #endif	/* defined(HASFDESCFS) && HASFDESCFS==1 */
 
+
+#if	defined(HAS_TMPFS)
+#define	_KERNEL
+#include <fs/tmpfs/tmpfs.h>
+#undef	_KERNEL
+#endif	/* defined(HAS_TMPFS) */
+
 _PROTOTYPE(static void get_lock_state,(KA_T f));
 
 
@@ -276,7 +283,9 @@ process_node(va)
 #  if	FREEBSDV<5000
 	struct specinfo si;
 #  else	/* FREEBSDV>=5000 */
+#   if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
 	struct cdev si;
+#   endif	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
 #  endif	/* FREEBSDV<5000 */
 # endif	/* FREEBSDV>=4000 */
 #endif	/* FREEBSDV>=2000 */
@@ -319,15 +328,23 @@ process_node(va)
 	struct ufs2_dinode d2;
 # endif	/* !defined(HAS_UFS1_2) */
 
+# if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
 	struct cdev cd;
+# endif	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+
 	int cds;
 	struct devfs_dirent de;
 	struct devfs_dirent *d;
 	char vtbuf[32];
 	char *vtbp;
 	enum vtagtype { VT_DEVFS, VT_FDESC, VT_ISOFS, VT_PSEUDOFS, VT_NFS,
-			VT_NULL, VT_UFS, VT_ZFS, VT_UNKNOWN
+			VT_NULL, VT_TMPFS, VT_UFS, VT_ZFS, VT_UNKNOWN
 		      };
+
+# if	defined(HAS_TMPFS)
+	struct tmpfs_node tn;
+	struct tmpfs_node *tnp;
+# endif	/* defined(HAS_TMPFS) */
 #endif	/* FREEBSDV>=5000 */
 
 #if	defined(HASNULLFS)
@@ -414,6 +431,11 @@ process_overlaid_node:
 	pnp = (struct pfs_node *)NULL;
 #endif	/* defined(HASPSEUDOFS) */
 
+# if	defined(HAS_TMPFS)
+	tnp = (struct tmpfs_node *)NULL;
+# endif	/* defined(HAS_TMPFS) */
+
+
 #if	defined(HAS_ZFS)
 	z = (zfs_info_t *)NULL;
 	zm = (char *)NULL;
@@ -478,6 +500,11 @@ process_overlaid_node:
 		else if (strcasecmp(vfs->typnm, "pseudofs") == 0)
 		    Ntype = N_PSEU;
 # endif	/* defined(HASPSEUDOFS) */
+
+# if	defined(HAS_TMPFS)
+		else if (strcasecmp(vfs->typnm, "tmpfs") == 0)
+		    Ntype = N_TMP;
+# endif	/* defined(HAS_TMPFS) */
 #endif	/* defined(MOUNT_NONE) */
 
 	    }
@@ -498,7 +525,11 @@ process_overlaid_node:
  */
 	if (((v->v_type == VCHR) || (v->v_type == VBLK))
 	&&  v->v_rdev
+
+# if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
 	&&  !kread((KA_T)v->v_rdev, (char *)&cd, sizeof(cd))
+# endif	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+
 	) {
 	    cds = 1;
 	}
@@ -538,6 +569,10 @@ process_overlaid_node:
 		vtag = VT_DEVFS;
 	    else if (!strcmp(vtbuf, "nfs"))
 		vtag = VT_NFS;
+	    else if (!strcmp(vtbuf, "newnfs"))
+		vtag = VT_NFS;
+	    else if (!strcmp(vtbuf, "oldnfs"))
+		vtag = VT_NFS;
 	    else if (!strcmp(vtbuf, "isofs"))
 		vtag = VT_ISOFS;
 	    else if (!strcmp(vtbuf, "pseudofs"))
@@ -546,6 +581,8 @@ process_overlaid_node:
 		vtag = VT_NULL;
 	    else if (!strcmp(vtbuf, "fdesc"))
 		vtag = VT_FDESC;
+	    else if (!strcmp(vtbuf, "tmpfs"))
+		vtag = VT_TMPFS;
 	} else
 	    vtbp = "(unknown)";
 #else	/* FREEBSDV<5000 */
@@ -729,6 +766,19 @@ process_overlaid_node:
 	    break;
 #endif	/* defined(HASPSEUDOFS) */
 
+# if	defined(HAS_TMPFS)
+	case VT_TMPFS:
+	    if (!v->v_data
+	    ||  kread((KA_T)v->v_data, (char *)&tn, sizeof(tn))) {
+		(void) snpf(Namech, Namechl, "no tmpfs_node: %s",
+		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
+		enter_nm(Namech);
+		return;
+	    }
+	    tnp = &tn;
+	    break;
+# endif	/* defined(HAS_TMPFS) */
+
 	case VT_UFS:
 
 #if	FREEBSDV<2000
@@ -770,8 +820,11 @@ process_overlaid_node:
 
 #if	defined(HAS_ZFS)
 	case VT_ZFS:
+	    memset((void *)&zi, 0, sizeof(zfs_info_t));
 	    if (!v->v_data
-	    ||  (zm = readzfsnode((KA_T)v->v_data, &zi))) {
+	    ||  (zm = readzfsnode((KA_T)v->v_data, &zi,
+				  ((v->v_vflag & VV_ROOT) ? 1 : 0)))
+	    ) {
 		(void) snpf(Namech, Namechl, "%s: %s", zm,
 		    print_kptr((KA_T)v->v_data, (char *)NULL, 0));
 		enter_nm(Namech);
@@ -818,10 +871,19 @@ process_overlaid_node:
 
 #if	FREEBSDV>=4000
 	    if (i->i_dev
-	    &&  !kread((KA_T)i->i_dev, (char *)&si, sizeof(si))) {
+
+# if	!defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV)
+	    &&  !kread((KA_T)i->i_dev, (char *)&si, sizeof(si))
+# endif	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+
+	    ) {
 
 # if	defined(HAS_NO_SI_UDEV)
-		dev = dev2udev(&si);
+#  if	defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
+		dev = Dev2Udev((KA_T)i->i_dev);
+#  else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+		dev = Dev2Udev(&si);
+#  endif	/* defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV) */
 # else	/* !defined(HAS_NO_SI_UDEV) */
 		dev = si.si_udev;
 # endif	/* defined(HAS_NO_SI_UDEV) */
@@ -849,7 +911,11 @@ process_overlaid_node:
 		if (cds) {
 
 # if	defined(HAS_NO_SI_UDEV)
-		    rdev = dev2udev(&cd);
+#  if	defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
+		    rdev = Dev2Udev((KA_T)v->v_rdev);
+#  else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+		    rdev = Dev2Udev(&cd);
+#  endif	/* defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV) */
 # else	/* !defined(HAS_NO_SI_UDEV) */
 		    rdev = cd.si_udev;
 # endif	/* defined(HAS_NO_SI_UDEV) */
@@ -927,7 +993,7 @@ process_overlaid_node:
 		dev = DevDev;
 		devs = 1;
 	    }
-	    if ((type == VCHR)) {
+	    if (type == VCHR) {
 
 # if	defined(HAS_UFS1_2)
 		if (ufst == 1) {
@@ -942,7 +1008,11 @@ process_overlaid_node:
 		if (cds) {
 
 # if	defined(HAS_NO_SI_UDEV)
-		    rdev = dev2udev(&cd);
+#  if	defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV)
+		    rdev = Dev2Udev((KA_T)v->v_rdev);
+#  else	/* !defined(HAS_CONF_MINOR) && !defined(HAS_CDEV2PRIV) */
+		    rdev = Dev2Udev(&cd);
+#  endif	/* defined(HAS_CONF_MINOR) || defined(HAS_CDEV2PRIV) */
 # else	/* !defined(HAS_NO_SI_UDEV) */
 		    rdev = cd.si_udev;
 # endif	/* defined(HAS_NO_SI_UDEV) */
@@ -962,6 +1032,19 @@ process_overlaid_node:
 	}
 #endif	/* defined(HASPSEUDOFS) */
 
+# if	defined(HAS_TMPFS)
+	else if (tnp) {
+	    if (vfs) {
+		dev = vfs->fsid.val[0];
+		devs = 1;
+	    }
+	    if (tnp->tn_type == VBLK || tnp->tn_type == VCHR) {
+		rdev = tnp->tn_rdev;
+		rdevs = 1;
+	    }
+	}
+# endif	/* defined(HAS_TMPFS) */
+
 /*
  * Obtain the inode number.
  */
@@ -1014,6 +1097,13 @@ process_overlaid_node:
 	}
 #endif	/* FREEBSDV>=5000 */
 
+# if	defined(HAS_TMPFS)
+	else if (tnp) {
+	    Lf->inode = (INODETYPE)tnp->tn_id;
+	    Lf->inp_ty = 1;
+	}
+# endif	/* defined(HAS_TMPFS) */
+
 /*
  * Obtain the file size.
  */
@@ -1120,7 +1210,20 @@ process_overlaid_node:
 		}
 		else if ((type == VCHR || type == VBLK) && !Fsize)
 		    Lf->off_def = 1;
+		break;
+
+# if	defined(HAS_TMPFS)
+		case N_TMP:
+		    if ((tnp->tn_type == VBLK || tnp->tn_type == VCHR)
+		    &&  !Fsize) {
+			Lf->off_def = 1;
+		    } else {
+			Lf->sz = (SZOFFTYPE)tnp->tn_size;
+			Lf->sz_def = 1;
+		    }
 		    break;
+# endif	/* defined(HAS_TMPFS) */
+
 	    }
 	}
 /*
@@ -1180,6 +1283,15 @@ process_overlaid_node:
 		break;
 #endif	/* defined(HASPSEUODOFS) */
 
+# if	defined(HAS_TMPFS)
+	    case N_TMP:
+		if (tnp) {
+		    Lf->nlink = (long)tnp->tn_links;
+		    Lf->nlink_def = 1;
+		}
+		break;
+# endif	/* defined(HAS_TMPFS) */
+
 	    }
 	    if (Lf->nlink_def && Nlink && (Lf->nlink < Nlink))
 		Lf->sf |= SELNLINK;
@@ -1238,8 +1350,8 @@ process_overlaid_node:
 	    ty = "FIFO";
 	    break;
 	default:
-	    (void) snpf(Lf->type, sizeof(Lf->type), "%04o", (type & 0xfff));
-	    ty = (char *)NULL;
+	     (void) snpf(Lf->type, sizeof(Lf->type), "%04o", (type & 0xfff));
+	     ty = (char *)NULL;
 	}
 	if (ty)
 	    (void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
