@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.7 99/05/09 14:55:42 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.19 2015/07/07 20:23:43 abe Exp $";
 #endif
 
 #include "lsof.h"
@@ -107,74 +107,145 @@ enter_vn_text(va, n)
 void
 gather_proc_info()
 {
+	short cckreg;			/* conditional status of regular file
+					 * checking:
+					 *     0 = unconditionally check
+					 *     1 = conditionally check */
+	short ckscko;			/* socket file only checking status:
+					 *     0 = none
+					 *     1 = check only socket files,
+					 *	   including TCP and UDP
+					 *	   streams with eXPORT data,
+					 *	   where supported */
 	struct filedesc fd;
 	int i, nf;
 	MALLOC_S nb;
-	static struct file **ofb = NULL;
+
+#if	defined(HAS_FILEDESCENT)
+	typedef struct filedescent ofb_t;
+#else	/* !defined(HAS_FILEDESCENT) */
+	typedef struct file* ofb_t;
+#endif	/* defined(HAS_FILEDESCENT) */
+
+#if	defined(HAS_FDESCENTTBL)
+	struct fdescenttbl fdt;
+	KA_T fa;
+#endif	/* defined(HAS_FDESCENTTBL) */
+
+	static ofb_t *ofb = NULL;
 	static int ofbb = 0;
-	int pgrp, pid;
+	int pgid, pid;
 	int ppid = 0;
 	short pss, sf;
 	int px;
+	int tid;			/* thread (task) ID */
 	uid_t uid;
 
-#if	FREEBSDV<200
+#if	FREEBSDV<2000
 	struct proc *p;
 	struct pcred pc;
 	struct pgrp pg;
-#else	/* FREEBSDV>=200 */
+#else	/* FREEBSDV>=2000 */
 	struct kinfo_proc *p;
-#endif	/* FREEBSDV<200 */
+#endif	/* FREEBSDV<2000 */
 
-#if	defined(HASFSTRUCT)
+#if	defined(HASFSTRUCT) && !defined(HAS_FILEDESCENT)
 	static char *pof = (char *)NULL;
 	static int pofb = 0;
-#endif	/* defined(HASFSTRUCT) */
+#endif	/* defined(HASFSTRUCT) && !defiled(HAS_FILEDESCENT) */
 
+/*
+ * Define socket and regular file conditional processing flags.
+ *
+ * If only socket files have been selected, or socket files have been
+ * selected, ANDed with other selection options, enable the skipping of
+ * regular files.
+ *
+ * If socket files and some process options have been selected, enable
+ * conditional skipping of regular file; i.e., regular files will be skipped
+ * unless they belong to a process selected by one of the specified options.
+ */
+	if (Selflags & SELNW) {
+
+	/*
+	 * Some network files selection options have been specified.
+	 */
+	    if (Fand || !(Selflags & ~SELNW)) {
+
+	    /*
+	     * Selection ANDing or only network file options have been
+	     * specified, so set unconditional skipping of regular files
+	     * and socket file only checking.
+	     */
+		cckreg = 0;
+		ckscko = 1;
+	    } else {
+
+	    /*
+	     * If ORed file selection options have been specified, or no
+	     * ORed process selection options have been specified, enable
+	     * unconditional file checking and clear socket file only
+	     * checking.
+	     *
+	     * If only ORed process selection options have been specified,
+	     * enable conditional file skipping and socket file only checking.
+	     */
+		if ((Selflags & SELFILE) || !(Selflags & SELPROC))
+		    cckreg = ckscko = 0;
+		else
+		    cckreg = ckscko = 1;
+	    }
+	} else {
+
+	/*
+	 * No network file selection options were specified.  Enable
+	 * unconditional file checking and clear socket file only checking.
+	 */
+	    cckreg = ckscko = 0;
+	}
 /*
  * Read the process table.
  */
 
-#if	FREEBSDV<200
+#if	FREEBSDV<2000
 	if ((Np = kvm_getprocs(KINFO_PROC_ALL, 0)) < 0)
-#else	/* FREEBSDV>=200 */
-	if ((P = kvm_getprocs(Kd, KERN_PROC_ALL, 0, &Np)) == NULL)
-#endif	/* FREEBSDV<200 */
+#else	/* FREEBSDV>=2000 */
+
+# if	!defined(KERN_PROC_PROC)
+#define	KERN_PROC_PROC  KERN_PROC_ALL
+# endif	/* !defined(KERN_PROC_PROC) */
+
+	if ((P = kvm_getprocs(Kd, Ftask ? KERN_PROC_ALL : KERN_PROC_PROC,
+			      0, &Np))
+	== NULL)
+#endif	/* FREEBSDV<2000 */
 
 	{
 	    (void) fprintf(stderr, "%s: can't read process table: %s\n",
 		Pn,
 
-#if	FREEBSDV<200
+#if	FREEBSDV<2000
 		kvm_geterr()
-#else	/* FREEBSDV>=200 */
+#else	/* FREEBSDV>=2000 */
 		kvm_geterr(Kd)
-#endif	/* FREEBSDV<200 */
+#endif	/* FREEBSDV<2000 */
 
 	    );
 	    Exit(1);
 	}
-
-#if	defined(HASNCACHE)
-/*
- * Read kernel name cache.
- */
-	ncache_load();
-#endif  /* defined(HASNCACHE) */
-
 /*
  * Examine proc structures and their associated information.
  */
 
-#if	FREEBSDV<200
+#if	FREEBSDV<2000
 	for (px = 0; px < Np; px++)
-#else	/* FREEBSDV>=200 */
+#else	/* FREEBSDV>=2000 */
 	for (p = P, px = 0; px < Np; p++, px++)
-#endif	/* FREEBSDV<200 */
+#endif	/* FREEBSDV<2000 */
 
 	{
 
-#if	FREEBSDV<200
+#if	FREEBSDV<2000
 	/*
 	 * Read process information, process group structure (if
 	 * necessary), and User ID (if necessary).
@@ -184,47 +255,88 @@ gather_proc_info()
 	    if (p->P_STAT == 0 || p->P_STAT == SZOMB)
 		continue;
 	    pg.pg_id = 0;
-	    if (Fpgrp && p->p_pgrp) {
-		if (kread((KA_T)p->p_pgrp, (char *)&pg, sizeof(pg)))
+	    if (Fpgid && p->P_PGID) {
+		if (kread((KA_T)p->P_PGID, (char *)&pg, sizeof(pg)))
 		    continue;
 	    }
-	    pgrp = pg.pg_id;
+	    pgid = pg.pg_id;
 	    if (!p->p_cred
 	    ||  kread((KA_T)p->p_cred, (char *)&pc, sizeof(pc)))
 		continue;
 	    uid = pc.p_ruid;
-#else	/* FREEBSDV>=200 */
+#else	/* FREEBSDV>=2000 */
 	    if (p->P_STAT == 0 || p->P_STAT == SZOMB)
 		continue;
-	    pgrp = p->P_PGRP;
+	    pgid = p->P_PGID;
+# if	FREEBSDV<5000
 	    uid = p->kp_eproc.e_ucred.cr_uid;
-#endif	/* FREEBSDV<200 */
+# else	/* FREEBSDV>=5000 */
+	    uid = p->ki_uid;
+# endif	/* FREEBSDV<5000 */
+#endif	/* FREEBSDV<2000 */
 
 #if	defined(HASPPID)
 	    ppid = p->P_PPID;
 #endif	/* defined(HASPPID) */
 
+#if	defined(HASTASKS)
+	/*
+	 * See if process,including its tasks, is excluded.
+	 */
+	    tid = Ftask ? (int)p->ki_tid : 0;
+	    if (is_proc_excl(p->P_PID, pgid, (UID_ARG)uid, &pss, &sf, tid))
+		continue;
+#else	/* !defined(HASTASKS) */
 	/*
 	 * See if process is excluded.
-	 *
+	 */
+	    if (is_proc_excl(p->P_PID, pgid, (UID_ARG)uid, &pss, &sf))
+		continue;
+#endif	/* defined(HASTASKS) */
+
+	/*
 	 * Read file structure pointers.
 	 */
-	    if (is_proc_excl(p->P_PID, pgrp, (UID_ARG)uid, &pss, &sf))
-		continue;
 	    if (!p->P_FD
 	    ||  kread((KA_T)p->P_FD, (char *)&fd, sizeof(fd)))
 		continue;
+
+#if	defined(HAS_FDESCENTTBL)
+	    if (!fd.fd_files
+	    ||  kread((KA_T)fd.fd_files, (char *)&fdt, sizeof(fdt)))
+		continue;
+	    if (!fd.fd_refcnt || fd.fd_lastfile > fdt.fdt_nfiles)
+		continue;
+#else	/* !defined(HAS_FDESCENTTBL) */
 	    if (!fd.fd_refcnt || fd.fd_lastfile > fd.fd_nfiles)
 		continue;
+#endif	/* defined(HAS_FDESCENTTBL) */
+
 	/*
 	 * Allocate a local process structure.
 	 */
 	    if (is_cmd_excl(p->P_COMM, &pss, &sf))
 		continue;
-	    alloc_lproc(p->P_PID, pgrp, ppid, (UID_ARG)uid, p->P_COMM,
+	    if (cckreg) {
+
+	    /*
+	     * If conditional checking of regular files is enabled, enable
+	     * socket file only checking, based on the process' selection
+	     * status.
+	     */
+		ckscko = (sf & SELPROC) ? 0 : 1;
+	    }
+	    alloc_lproc(p->P_PID, pgid, ppid, (UID_ARG)uid, p->P_COMM,
 		(int)pss, (int)sf);
 	    Plf = (struct lfile *)NULL;
 
+#if	defined(HASTASKS)
+	/*
+	 * Save the task (thread) ID.
+	 */
+	    Lp->tid = tid;
+#endif	/* defined(HASTASKS) */
+
 #if	defined(P_ADDR)
 	/*
 	 * Save the kernel proc struct address, if P_ADDR is defined.
@@ -235,7 +347,7 @@ gather_proc_info()
 	/*
 	 * Save current working directory information.
 	 */
-	    if (fd.fd_cdir) {
+	    if (!ckscko && fd.fd_cdir) {
 		alloc_lfile(CWD, -1);
 		Cfp = (struct file *)NULL;
 		process_node((KA_T)fd.fd_cdir);
@@ -245,29 +357,50 @@ gather_proc_info()
 	/*
 	 * Save root directory information.
 	 */
-	    if (fd.fd_rdir) {
+	    if (!ckscko && fd.fd_rdir) {
 		alloc_lfile(RTD, -1);
 		Cfp = (struct file *)NULL;
 		process_node((KA_T)fd.fd_rdir);
 		if (Lf->sf)
 		    link_lfile();
 	    }
+
+#if	FREEBSDV>=5000
+	/*
+	 * Save jail directory information.
+	 */
+	    if (!ckscko && fd.fd_jdir) {
+		alloc_lfile("jld", -1);
+		Cfp = (struct file *)NULL;
+		process_node((KA_T)fd.fd_jdir);
+		if (Lf->sf)
+		    link_lfile();
+	    }
+#endif	/* FREEBSDV>=5000 */
+
 	/*
 	 * Save information on the text file.
 	 */
-	    if (p->P_VMSPACE)
+	    if (!ckscko && p->P_VMSPACE)
 		process_text((KA_T)p->P_VMSPACE);
 	/*
 	 * Read open file structure pointers.
 	 */
+
+#if	defined(HAS_FDESCENTTBL)
+	    if ((nf = fdt.fdt_nfiles) <= 0)
+		continue;
+#else	/* !defined(HAS_FDESCENTTBL) */
 	    if (!fd.fd_ofiles || (nf = fd.fd_nfiles) <= 0)
 		continue;
-	    nb = (MALLOC_S)(sizeof(struct file *) * nf);
+#endif	/* defined(HAS_FDESCENTTBL) */
+
+	    nb = (MALLOC_S)(sizeof(ofb_t) * nf);
 	    if (nb > ofbb) {
 		if (!ofb)
-		    ofb = (struct file **)malloc(nb);
+		    ofb = (ofb_t *)malloc(nb);
 		else
-		    ofb = (struct file **)realloc((MALLOC_P *)ofb, nb);
+		    ofb = (ofb_t *)realloc((MALLOC_P *)ofb, nb);
 		if (!ofb) {
 		    (void) fprintf(stderr, "%s: PID %d, no file * space\n",
 			Pn, p->P_PID);
@@ -275,10 +408,19 @@ gather_proc_info()
 		}
 		ofbb = nb;
 	    }
+
+#if	defined(HAS_FDESCENTTBL)
+	    fa = (KA_T)fd.fd_files
+	       + (KA_T)offsetof(struct fdescenttbl, fdt_ofiles);
+	    if (kread(fa, (char *)ofb, nb))
+		continue;
+#else	/* !defined(HAS_FDESCENTTBL) */
 	    if (kread((KA_T)fd.fd_ofiles, (char *)ofb, nb))
 		continue;
+#endif	/* defined(HAS_FDESCENTTBL) */
 
-#if	defined(HASFSTRUCT)
+
+#if	defined(HASFSTRUCT) && !defined(HAS_FILEDESCENT)
 	    if (Fsv & FSV_FG) {
 		nb = (MALLOC_S)(sizeof(char) * nf);
 		if (nb > pofb) {
@@ -296,20 +438,31 @@ gather_proc_info()
 		if (!fd.fd_ofileflags || kread((KA_T)fd.fd_ofileflags, pof, nb))
 		    zeromem(pof, nb);
 	    }
-#endif	/* defined(HASFSTRUCT) */
+#endif	/* defined(HASFSTRUCT) && !defined(HAS_FILEDESCENT) */
 
 	/*
 	 * Save information on file descriptors.
 	 */
 	    for (i = 0; i < nf; i++) {
-		if (ofb[i]) {
+
+#if	defined(HAS_FILEDESCENT)
+		if ((Cfp = ofb[i].fde_file))
+#else	/* !defined(HAS_FILEDESCENT) */
+		if ((Cfp = ofb[i]))
+#endif	/* defined(HAS_FILEDESCENT) */
+
+		{
 		    alloc_lfile(NULL, i);
-		    process_file((KA_T)(Cfp = ofb[i]));
+		    process_file((KA_T)Cfp);
 		    if (Lf->sf) {
 
 #if	defined(HASFSTRUCT)
 			if (Fsv & FSV_FG)
+# if	defined(HAS_FILEDESCENT)
+			    Lf->pof = (long)ofb[i].fde_flags;
+# else	/* !defined(HAS_FILEDESCENT) */
 			    Lf->pof = (long)pof[i];
+# endif	/* defined(HAS_FILEDESCENT) */
 #endif	/* defined(HASFSTRUCT) */
 
 			link_lfile();
@@ -317,10 +470,12 @@ gather_proc_info()
 		}
 	    }
 	/*
-	 * Examine results.
+	 * Unless threads (tasks) are being processed, examine results.
 	 */
-	    if (examine_lproc())
-		return;
+	    if (!Ftask) {
+		if (examine_lproc())
+		    return;
+	    }
 	}
 }
 
@@ -374,30 +529,37 @@ get_kernel_access()
  * Open kernel memory access.
  */
 
-#if	FREEBSDV<200
+#if	FREEBSDV<2000
 	if (kvm_openfiles(Nmlst, Memory, NULL) == -1)
-#else	/* FREEBSDV>=200 */
+#else	/* FREEBSDV>=2000 */
 	if ((Kd = kvm_open(Nmlst, Memory, NULL, O_RDONLY, NULL)) == NULL)
-#endif	/* FREEBSDV<200 */
+#endif	/* FREEBSDV<2000 */
 
 	{
 	    (void) fprintf(stderr,
-		"%s: kvm_open%s (namelist=%s, core = %s): %s\n",
+		"%s: kvm_open%s(execfile=%s, corefile=%s): %s\n",
 		Pn,
 
-#if	FREEBSDV<200
+#if	FREEBSDV<2000
 		"files",
-#else	/* FREEBSDV>=200 */
+#else	/* FREEBSDV>=2000 */
 		"",
-#endif	/* FREEBSDV<200 */
+#endif	/* FREEBSDV<2000 */
 
 		Nmlst ? Nmlst : "default",
-		Memory  ? Memory  : "default",
+		Memory ? Memory :
+
+#if	defined(_PATH_MEM)
+				  _PATH_MEM,
+#else	/* !defined(_PATH_MEM) */
+				  "default",
+#endif	/* defined(_PATH_MEM) */
+
 		strerror(errno));
 	    Exit(1);
 	}
 	(void) build_Nl(Drive_Nl);
-	if (nlist(Nmlst, Nl) < 0) {
+	if (kvm_nlist(Kd, Nl) < 0) {
 	    (void) fprintf(stderr, "%s: can't read namelist from %s\n",
 		Pn, Nmlst);
 	    Exit(1);
@@ -439,10 +601,10 @@ get_nlist_path(ap)
 	    if (!(bfc = (char *)malloc(bfl))) {
 		(void) fprintf(stderr,
 		    "%s: can't allocate %d bytes for boot file path: %s\n",
-		    Pn, bfl, bf);
+		    Pn, (int)bfl, bf);
 		Exit(1);
 	    }
-	    (void) strcpy(bfc, bf);
+	    (void) snpf(bfc, bfl, "%s", bf);
 	    return(bfc);
 	}
 	return((char *)NULL);
@@ -473,11 +635,11 @@ kread(addr, buf, len)
 {
 	int br;
 
-#if	FREEBSDV<200
+#if	FREEBSDV<2000
 	br = kvm_read((void *)addr, (void *)buf, len);
-#else	/* FREEBSDV>=200 */
+#else	/* FREEBSDV>=2000 */
 	br = kvm_read(Kd, (u_long)addr, buf, len);
-#endif	/* FREEBSDV<200 */
+#endif	/* FREEBSDV<2000 */
 
 	return((br == len) ? 0 : 1);
 }
@@ -497,9 +659,9 @@ process_text(vm)
 	struct vm_object vmo;
 	struct vmspace vmsp;
 
-#if	FREEBSDV<220
+#if	FREEBSDV<2020
 	struct pager_struct pg;
-#endif	/* FREEBSDV<220 */
+#endif	/* FREEBSDV<2020 */
 
 /*
  * Read the vmspace structure for the process.
@@ -540,29 +702,29 @@ process_text(vm)
 		 j < 2 && ka;
 		 j++,
 
-#if	FREEBSDV<220
+#if	FREEBSDV<2020
 		 ka = (KA_T)vmo.shadow
-#else	/* FREEBSDV>=220 */
+#else	/* FREEBSDV>=2020 */
 		 ka = (KA_T)vmo.backing_object
-#endif	/* FREEBSDV<220 */
+#endif	/* FREEBSDV<2020 */
 		 )
 	    {
 		if (kread(ka, (char *)&vmo, sizeof(vmo)))
 		    break;
 
-#if	FREEBSDV<220
+#if	FREEBSDV<2020
 		if ((ka = (KA_T)vmo.pager) == NULL
 		||  kread(ka, (char *)&pg, sizeof(pg)))
 		    continue;
 		if (pg.pg_handle == NULL || pg.pg_type != PG_VNODE)
 		    continue;
 		(void) (enter_vn_text((KA_T)pg.pg_handle, &n));
-#else	/* FREEBSDV>=220 */
+#else	/* FREEBSDV>=2020 */
 		if (vmo.type != OBJT_VNODE
 		||  vmo.handle == (void *)NULL)
 		    continue;
 		(void) (enter_vn_text((KA_T)vmo.handle, &n));
-#endif	/* FREEBSDV<220 */
+#endif	/* FREEBSDV<2020 */
 
 	    }
 	}
