@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.7 2006/03/28 20:45:24 abe Exp $
+ * $Id: dlsof.h,v 1.8 2008/10/21 16:17:50 abe Exp $
  */
 
 
@@ -67,6 +67,16 @@
 #undef	_APP32_64BIT_OFF_T
 # endif
 
+# if	!defined(__LP64__) && defined(_LARGEFILE64_SOURCE) && HPUXV>=1123
+/*
+ * Make sure a 32 bit lsof for HPUX>=1123 uses [l]stat64 when
+ * _LARGEFILE64_SOURCE is defined.
+ */
+
+#define	stat	stat64
+#define	lstat	lstat64
+# endif	/* !defined(__LP64__) && defined(_LARGEFILE64_SOURCE) && HPUXV>=1123 */
+
 #include <sys/socket.h>
 
 # if	defined(_FILE_OFFSET_BITS) && _FILE_OFFSET_BITS==64 && defined(_APP32_64BIT_OFF_T)
