@@ -43,11 +43,17 @@ static char *rcsid = "$Id";
  * Local function prototypes
  */
 
+#if	defined(PS_STR_XPORT_DATA)
+_PROTOTYPE(static void make_sock,(struct pst_fileinfo2 *f,
+				  struct pst_stream *sh,
+				  struct pst_socket *s));
+#endif	/* defined(PS_STR_XPORT_DATA) */
+
 _PROTOTYPE(static void printpsproto,(uint32_t p));
 
 
 /*
- * Local macroes
+ * Local macros
  */
 
 #if	defined(HASIPv6)
@@ -61,6 +67,395 @@ _PROTOTYPE(static void printpsproto,(uint32_t p));
 #endif	/* defined(HASIPv6) */
 
 
+/*
+ * build_IPstates() -- build the TCP and UDP state tables
+ */
+
+void
+build_IPstates()
+{
+	if (!TcpSt) {
+	    (void) enter_IPstate("TCP", "CLOSED", PS_TCPS_CLOSED);
+	    (void) enter_IPstate("TCP", "IDLE", PS_TCPS_IDLE);
+	    (void) enter_IPstate("TCP", "BOUND", PS_TCPS_BOUND);
+	    (void) enter_IPstate("TCP", "LISTEN", PS_TCPS_LISTEN);
+	    (void) enter_IPstate("TCP", "SYN_SENT", PS_TCPS_SYN_SENT);
+	    (void) enter_IPstate("TCP", "SYN_RCVD", PS_TCPS_SYN_RCVD);
+	    (void) enter_IPstate("TCP", "ESTABLISHED", PS_TCPS_ESTABLISHED);
+	    (void) enter_IPstate("TCP", "CLOSE_WAIT", PS_TCPS_CLOSE_WAIT);
+	    (void) enter_IPstate("TCP", "FIN_WAIT_1", PS_TCPS_FIN_WAIT_1);
+	    (void) enter_IPstate("TCP", "CLOSING", PS_TCPS_CLOSING);
+	    (void) enter_IPstate("TCP", "LAST_ACK", PS_TCPS_LAST_ACK);
+	    (void) enter_IPstate("TCP", "FIN_WAIT_2", PS_TCPS_FIN_WAIT_2);
+	    (void) enter_IPstate("TCP", "TIME_WAIT", PS_TCPS_TIME_WAIT);
+	    (void) enter_IPstate("TCP", (char *)NULL, 0);
+	}
+	if (!UdpSt) {
+	    (void) enter_IPstate("UDP", "Uninitialized", PS_TS_UNINIT);
+	    (void) enter_IPstate("UDP", "Unbound", PS_TS_UNBND);
+	    (void) enter_IPstate("UDP", "Wait_BIND_REQ_Ack", PS_TS_WACK_BREQ);
+	    (void) enter_IPstate("UDP", "Wait_UNBIND_REQ_Ack", PS_TS_WACK_UREQ);
+	    (void) enter_IPstate("UDP", "Idle", PS_TS_IDLE);
+	    (void) enter_IPstate("UDP", "Wait_OPT_REQ_Ack", PS_TS_WACK_OPTREQ);
+	    (void) enter_IPstate("UDP", "Wait_CONN_REQ_Ack", PS_TS_WACK_CREQ);
+	    (void) enter_IPstate("UDP", "Wait_CONN_REQ_Confirm",
+		PS_TS_WCON_CREQ);
+	    (void) enter_IPstate("UDP", "Wait_CONN_IND_Response",
+		PS_TS_WRES_CIND);
+	    (void) enter_IPstate("UDP", "Wait_CONN_RES_Ack", PS_TS_WACK_CRES);
+	    (void) enter_IPstate("UDP", "Wait_Data_Xfr", PS_TS_DATA_XFER);
+	    (void) enter_IPstate("UDP", "Wait_Read_Release", PS_TS_WIND_ORDREL);
+	    (void) enter_IPstate("UDP", "Wait_Write_Release",
+		PS_TS_WREQ_ORDREL);
+	    (void) enter_IPstate("UDP", "Wait_DISCON_REQ_Ack",
+		PS_TS_WACK_DREQ6);
+	    (void) enter_IPstate("UDP", "Wait_DISCON_REQ_Ack",
+		PS_TS_WACK_DREQ7);
+	    (void) enter_IPstate("UDP", "Wait_DISCON_REQ_Ack",
+		PS_TS_WACK_DREQ9);
+	    (void) enter_IPstate("UDP", "Wait_DISCON_REQ_Ack",
+		PS_TS_WACK_DREQ10);
+	    (void) enter_IPstate("UDP", "Wait_DISCON_REQ_Ack",
+		PS_TS_WACK_DREQ11);
+	    (void) enter_IPstate("UDP", "Internal", PS_TS_WACK_ORDREL);
+	    (void) enter_IPstate("UDP", (char *)NULL, 0);
+	}
+}
+
+
+#if	defined(PS_STR_XPORT_DATA)
+/*
+ * make_sock() -- make a socket from the eXPORT data in a stream's head
+ */
+
+static void
+make_sock(f, sh, s)
+	struct pst_fileinfo2 *f;		/* pst_fileinfo2 */
+	struct pst_stream *sh;			/* stream head */
+	struct pst_socket *s;			/* constructed socket */
+{
+	size_t sz;
+/*
+ * Zero the destination pst_socket structure  and propagate its file and node
+ * IDs from the stream head.  Also propagate the linger time. 
+ */
+	(void)memset((void *)s, 0, sizeof(struct pst_socket));
+	s->pst_hi_fileid = sh->val.head.pst_hi_fileid;
+	s->pst_lo_fileid = sh->val.head.pst_lo_fileid;
+	s->pst_hi_nodeid = sh->val.head.pst_hi_nodeid;
+	s->pst_lo_nodeid = sh->val.head.pst_lo_nodeid;
+	s->pst_linger = sh->pst_str_xport_linger;
+/*
+ * Convert stream family to socket family and stream protocol to socket
+ * protocol.
+ *
+ * This could be avoided if PSTAT were to use a common set of family and
+ * protocol symbols.
+ */
+	switch (sh->pst_str_xport_family) {
+	    case PS_STR_XPORT_AFINET:
+		s->pst_family = PS_AF_INET;
+		break;
+	    case PS_STR_XPORT_AFINET6:
+		s->pst_family = PS_AF_INET6;
+		break;
+	    default:
+		s->pst_family = sh->pst_str_xport_family;
+	}
+	switch (sh->pst_str_xport_protocol) {
+	    case PS_STR_XPORT_TCP_PROTO:
+		s->pst_protocol = PS_PROTO_TCP;
+		break;
+	    case PS_STR_XPORT_UDP_PROTO:
+		s->pst_protocol = PS_PROTO_UDP;
+		break;
+	    default:
+		s->pst_protocol = sh->pst_str_xport_protocol;
+	}
+/*
+ * Copy stream size information.
+ */
+	s->pst_qlimit = sh->pst_str_xport_qlimit;
+	s->pst_qlen = sh->pst_str_xport_qlen;
+	s->pst_idata = sh->pst_str_xport_idata;
+	s->pst_ibufsz = sh->pst_str_xport_ibufsz;
+	s->pst_rwnd = sh->pst_str_xport_rwnd;
+	s->pst_swnd = sh->pst_str_xport_swnd;
+	s->pst_odata = sh->pst_str_xport_odata;
+	s->pst_obufsz = sh->pst_str_xport_obufsz;
+/*
+ * Propagate protocol state from stream symbol values to socket ones.
+ *
+ * This could be avoided if PSTAT were to use a common set of protocol
+ * state symbols.
+ */
+	if (s->pst_protocol == PS_PROTO_TCP) {
+	    switch (sh->pst_str_xport_pstate) {
+
+#if	defined(PS_STR_XPORT_TCPS_CLOSED) && defined(PS_TCPS_CLOSED) \
+	&& (PS_STR_XPORT_TCPS_CLOSED != PS_TCPS_CLOSED)
+	    case PS_STR_XPORT_TCPS_CLOSED:
+		s->pst_pstate = PS_TCPS_CLOSED;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TCPS_IDLE) && defined(PS_TCPS_IDLE) \
+	&& (PS_STR_XPORT_TCPS_IDLE != PS_TCPS_IDLE)
+	    case PS_STR_XPORT_TCPS_IDLE:
+		s->pst_pstate = PS_TCPS_IDLE;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TCPS_BOUND) && defined(PS_TCPS_BOUND) \
+	&& (PS_STR_XPORT_TCPS_BOUND != PS_TCPS_BOUND)
+	    case PS_STR_XPORT_TCPS_BOUND:
+		s->pst_pstate = PS_TCPS_BOUND;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TCPS_LISTEN) && defined(PS_TCPS_LISTEN) \
+	&& (PS_STR_XPORT_TCPS_LISTEN != PS_TCPS_LISTEN)
+	    case PS_STR_XPORT_TCPS_LISTEN:
+		s->pst_pstate = PS_TCPS_LISTEN;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TCPS_SYN_SENT) && defined(PS_TCPS_SYN_SENT) \
+	&& (PS_STR_XPORT_TCPS_SYN_SENT != PS_TCPS_SYN_SENT)
+	    case PS_STR_XPORT_TCPS_SYN_SENT:
+		s->pst_pstate = PS_TCPS_SYN_SENT;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TCPS_SYN_RCVD) && defined(PS_TCPS_SYN_RCVD) \
+	&& (PS_STR_XPORT_TCPS_SYN_RCVD != PS_TCPS_SYN_RCVD)
+	    case PS_STR_XPORT_TCPS_SYN_RCVD:
+		s->pst_pstate = PS_TCPS_SYN_RCVD;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TCPS_ESTABLISHED) && defined(PS_TCPS_ESTABLISHED) \
+	&& (PS_STR_XPORT_TCPS_ESTABLISHED != PS_TCPS_ESTABLISHED)
+	    case PS_STR_XPORT_TCPS_ESTABLISHED:
+		s->pst_pstate = PS_TCPS_ESTABLISHED;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TCPS_CLOSE_WAIT) && defined(PS_TCPS_CLOSE_WAIT) \
+	&& (PS_STR_XPORT_TCPS_CLOSE_WAIT != PS_TCPS_CLOSE_WAIT)
+	    case PS_STR_XPORT_TCPS_CLOSE_WAIT:
+		s->pst_pstate = PS_TCPS_CLOSE_WAIT;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TCPS_FIN_WAIT_1) && defined(PS_TCPS_FIN_WAIT_1) \
+	&& (PS_STR_XPORT_TCPS_FIN_WAIT_1 != PS_TCPS_FIN_WAIT_1)
+	    case PS_STR_XPORT_TCPS_FIN_WAIT_1:
+		s->pst_pstate = PS_TCPS_FIN_WAIT_1;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TCPS_CLOSING) && defined(PS_TCPS_CLOSING) \
+	&& (PS_STR_XPORT_TCPS_CLOSING != PS_TCPS_CLOSING)
+	    case PS_STR_XPORT_TCPS_CLOSING:
+		s->pst_pstate = PS_TCPS_CLOSING;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TCPS_LAST_ACK) && defined(PS_TCPS_LAST_ACK) \
+	&& (PS_STR_XPORT_TCPS_LAST_ACK != PS_TCPS_LAST_ACK)
+	    case PS_STR_XPORT_TCPS_LAST_ACK:
+		s->pst_pstate = PS_TCPS_LAST_ACK;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TCPS_FIN_WAIT_2) && defined(PS_TCPS_FIN_WAIT_2) \
+	&& (PS_STR_XPORT_TCPS_FIN_WAIT_2 != PS_TCPS_FIN_WAIT_2)
+	    case PS_STR_XPORT_TCPS_FIN_WAIT_2:
+		s->pst_pstate = PS_TCPS_FIN_WAIT_2;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TCPS_TIME_WAIT) && defined(PS_TCPS_TIME_WAIT) \
+	&& (PS_STR_XPORT_TCPS_TIME_WAIT != PS_TCPS_TIME_WAIT)
+	    case PS_STR_XPORT_TCPS_TIME_WAIT:
+		s->pst_pstate = PS_TCPS_TIME_WAIT;
+		break;
+#endif
+
+	    default:
+		s->pst_pstate = sh->pst_str_xport_pstate;
+	    }
+	} else if (s->pst_protocol == PS_PROTO_UDP) {
+	    switch (sh->pst_str_xport_pstate) {
+
+#if	defined(PS_STR_XPORT_TS_UNINIT) && defined(PS_TS_UNINIT) \
+	&& (PS_STR_XPORT_TS_UNINIT != PS_TS_UNINIT)
+	    case PS_STR_XPORT_TS_UNINIT:
+		s->pst_pstate = PS_TS_UNINIT;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_UNBND) && defined(PS_TS_UNBND) \
+	&& (PS_STR_XPORT_TS_UNBND != PS_TS_UNBND)
+	    case PS_STR_XPORT_TS_UNBND:
+		s->pst_pstate = PS_TS_UNBND;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WACK_BREQ) && defined(PS_TS_WACK_BREQ) \
+	&& (PS_STR_XPORT_TS_WACK_BREQ != PS_TS_WACK_BREQ)
+	    case PS_STR_XPORT_TS_WACK_BREQ:
+		s->pst_pstate = PS_TS_WACK_BREQ;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WACK_UREQ) && defined(PS_TS_WACK_UREQ) \
+	&& (PS_STR_XPORT_TS_WACK_UREQ != PS_TS_WACK_UREQ)
+	    case PS_STR_XPORT_TS_WACK_UREQ:
+		s->pst_pstate = PS_TS_WACK_UREQ;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_IDLE) && defined(PS_TS_IDLE) \
+	&& (PS_STR_XPORT_TS_IDLE != PS_TS_IDLE)
+	    case PS_STR_XPORT_TS_IDLE:
+		s->pst_pstate = PS_TS_IDLE;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WACK_OPTREQ) && defined(PS_TS_WACK_OPTREQ) \
+	&& (PS_STR_XPORT_TS_WACK_OPTREQ != PS_TS_WACK_OPTREQ)
+	    case PS_STR_XPORT_TS_WACK_OPTREQ:
+		s->pst_pstate = PS_TS_WACK_OPTREQ;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WACK_CREQ) && defined(PS_TS_WACK_CREQ) \
+	&& (PS_STR_XPORT_TS_WACK_CREQ != PS_TS_WACK_CREQ)
+	    case PS_STR_XPORT_TS_WACK_CREQ:
+		s->pst_pstate = PS_TS_WACK_CREQ;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WCON_CREQ) && defined(PS_TS_WCON_CREQ) \
+	&& (PS_STR_XPORT_TS_WCON_CREQ != PS_TS_WCON_CREQ)
+	    case PS_STR_XPORT_TS_WCON_CREQ:
+		s->pst_pstate = PS_TS_WCON_CREQ;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WRES_CIND) && defined(PS_TS_WRES_CIND) \
+	&& (PS_STR_XPORT_TS_WRES_CIND != PS_TS_WRES_CIND)
+	    case PS_STR_XPORT_TS_WRES_CIND:
+		s->pst_pstate = PS_TS_WRES_CIND;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WACK_CRES) && defined(PS_TS_WACK_CRES) \
+	&& (PS_STR_XPORT_TS_WACK_CRES != PS_TS_WACK_CRES)
+	    case PS_STR_XPORT_TS_WACK_CRES:
+		s->pst_pstate = PS_TS_WACK_CRES;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_DATA_XFER) && defined(PS_TS_DATA_XFER) \
+	&& (PS_STR_XPORT_TS_DATA_XFER != PS_TS_DATA_XFER)
+	    case PS_STR_XPORT_TS_DATA_XFER:
+		s->pst_pstate = PS_TS_DATA_XFER;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WIND_ORDREL) && defined(PS_TS_WIND_ORDREL) \
+	&& (PS_STR_XPORT_TS_WIND_ORDREL != PS_TS_WIND_ORDREL)
+	    case PS_STR_XPORT_TS_WIND_ORDREL:
+		s->pst_pstate = PS_TS_WIND_ORDREL;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WREQ_ORDREL) && defined(PS_TS_WREQ_ORDREL) \
+	&& (PS_STR_XPORT_TS_WREQ_ORDREL != PS_TS_WREQ_ORDREL)
+	    case PS_STR_XPORT_TS_WREQ_ORDREL:
+		s->pst_pstate = PS_TS_WREQ_ORDREL;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WACK_DREQ6) && defined(PS_TS_WACK_DREQ6) \
+	&& (PS_STR_XPORT_TS_WACK_DREQ6 != PS_TS_WACK_DREQ6)
+	    case PS_STR_XPORT_TS_WACK_DREQ6:
+		s->pst_pstate = PS_TS_WACK_DREQ6;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WACK_DREQ7) && defined(PS_TS_WACK_DREQ7) \
+	&& (PS_STR_XPORT_TS_WACK_DREQ7 != PS_TS_WACK_DREQ7)
+	    case PS_STR_XPORT_TS_WACK_DREQ7:
+		s->pst_pstate = PS_TS_WACK_DREQ7;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WACK_DREQ9) && defined(PS_TS_WACK_DREQ9) \
+	&& (PS_STR_XPORT_TS_WACK_DREQ9 != PS_TS_WACK_DREQ9)
+	    case PS_STR_XPORT_TS_WACK_DREQ9:
+		s->pst_pstate = PS_TS_WACK_DREQ9;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WACK_DREQ10) && defined(PS_TS_WACK_DREQ10) \
+	&& (PS_STR_XPORT_TS_WACK_DREQ10 != PS_TS_WACK_DREQ10)
+	    case PS_STR_XPORT_TS_WACK_DREQ10:
+		s->pst_pstate = PS_TS_WACK_DREQ10;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WACK_DREQ11) && defined(PS_TS_WACK_DREQ11) \
+	&& (PS_STR_XPORT_TS_WACK_DREQ11 != PS_TS_WACK_DREQ11)
+	    case PS_STR_XPORT_TS_WACK_DREQ11:
+		s->pst_pstate = PS_TS_WACK_DREQ11;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_WACK_ORDREL) && defined(PS_TS_WACK_ORDREL) \
+	&& (PS_STR_XPORT_TS_WACK_ORDREL != PS_TS_WACK_ORDREL)
+	    case PS_STR_XPORT_TS_WACK_ORDREL:
+		s->pst_pstate = PS_TS_WACK_ORDREL;
+		break;
+#endif
+
+#if	defined(PS_STR_XPORT_TS_NOSTATES) && defined(PS_TS_NOSTATES) \
+	&& (PS_STR_XPORT_TS_NOSTATES != PS_TS_NOSTATES)
+	    case PS_STR_XPORT_TS_NOSTATES:
+		s->pst_pstate = PS_TS_NOSTATES;
+		break;
+#endif
+
+	    default:
+		s->pst_pstate = sh->pst_str_xport_pstate;
+	    }
+	} else
+	    s->pst_pstate = sh->pst_str_xport_pstate;
+/*
+ * Now propagate the bound and remote address information from pst_stream
+ * to the pst_socket structure.  Validate the copy lengths.
+ */
+	sz = (size_t)sh->pst_str_xport_boundaddr_len;
+	if (sz > sizeof(s->pst_boundaddr))
+	    sz = sizeof(s->pst_boundaddr);
+	if ((s->pst_boundaddr_len = sz)) {
+	    (void) memcpy((void *)s->pst_boundaddr,
+			  (const void *)sh->pst_str_xport_boundaddr, sz);
+	}
+	sz = (size_t)sh->pst_str_xport_remaddr_len;
+	if (sz > sizeof(s->pst_remaddr))
+	    sz = sizeof(s->pst_remaddr);
+	if ((s->pst_remaddr_len = sz)) {
+	    (void) memcpy((void *)s->pst_remaddr,
+			  (const void *)sh->pst_str_xport_remaddr, sz);
+	}
+}
+#endif	/* defined(PS_STR_XPORT_DATA) */
+
+
 /*
  * printpsproto() -- print PSTAT protocol name
  */
@@ -167,114 +562,34 @@ print_tcptpi(nl)
 	if (Ftcptpi & TCPTPI_STATE) {
 	    switch (Lf->lts.type) {
 	    case 0:				/* TCP */
-		switch ((i = Lf->lts.state.i)) {
-		case PS_TCPS_CLOSED:
-		    cp = "CLOSED";
-		    break;
-		case PS_TCPS_IDLE:
-		    cp = "IDLE";
-		    break;
-		case PS_TCPS_BOUND:
-		    cp = "BOUND";
-		    break;
-		case PS_TCPS_LISTEN:
-		    cp = "LISTEN";
-		    break;
-		case PS_TCPS_SYN_SENT:
-		    cp = "SYN_SENT";
-		    break;
-		case PS_TCPS_SYN_RCVD:
-		    cp = "SYN_RCVD";
-		    break;
-		case PS_TCPS_ESTABLISHED:
-		    cp = "ESTABLISHED";
-		    break;
-		case PS_TCPS_CLOSE_WAIT:
-		    cp = "CLOSE_WAIT";
-		    break;
-		case PS_TCPS_FIN_WAIT_1:
-		    cp = "FIN_WAIT_1";
-		    break;
-		case PS_TCPS_CLOSING:
-		    cp = "CLOSING";
-		    break;
-		case PS_TCPS_LAST_ACK:
-		    cp = "LAST_ACK";
-		    break;
-		case PS_TCPS_FIN_WAIT_2:
-		    cp = "FIN_WAIT_2";
-		    break;
-		case PS_TCPS_TIME_WAIT:
-		    cp = "TIME_WAIT";
-		    break;
-		default:
-		    (void) snpf(sbuf, sizeof(sbuf), "UknownState_%d", i);
+		if (!TcpSt)
+		    (void) build_IPstates();
+		if ((i = Lf->lts.state.i + TcpStOff) < 0 || i >= TcpNstates) {
+		    (void) snpf(sbuf, sizeof(sbuf), "UknownState_%d",
+			Lf->lts.state.i);
 		    cp = sbuf;
-		}
+		} else
+		    cp = TcpSt[i];
 		break;
-	    case 1:				/* TPI */
-		switch ((u = Lf->lts.state.ui)) {
-		case PS_TS_UNINIT:
-		    cp = "Uninitialized";
-		    break;
-		case PS_TS_UNBND:
-		    cp = "Unbound";
-		    break;
-		case PS_TS_WACK_BREQ:
-		    cp = "Wait_BIND_REQ_Ack";
-		    break;
-		case PS_TS_WACK_UREQ:
-		    cp = "Wait_UNBIND_REQ_Ack";
-		    break;
-		case PS_TS_IDLE:
-		    cp = "Idle";
-		    break;
-		case PS_TS_WACK_OPTREQ:
-		    cp = "Wait_OPT_REQ_Ack";
-		    break;
-		case PS_TS_WACK_CREQ:
-		    cp = "Wait_CONN_REQ_Ack";
-		    break;
-		case PS_TS_WCON_CREQ:
-		    cp = "Wait_CONN_REQ_Confirm";
-		    break;
-		case PS_TS_WRES_CIND:
-		    cp = "Wait_CONN_IND_Response";
-		    break;
-		case PS_TS_WACK_CRES:
-		    cp = "Wait_CONN_RES_Ack";
-		    break;
-		case PS_TS_DATA_XFER:
-		    cp = "Wait_Data_Xfr";
-		    break;
-		case PS_TS_WIND_ORDREL:
-		    cp = "Wait_Read_Release";
-		    break;
-		case PS_TS_WREQ_ORDREL:
-		    cp = "Wait_Write_Release";
-		    break;
-		case PS_TS_WACK_DREQ6:
-		case PS_TS_WACK_DREQ7:
-		case PS_TS_WACK_DREQ9:
-		case PS_TS_WACK_DREQ10:
-		case PS_TS_WACK_DREQ11:
-		    cp = "Wait_DISCON_REQ_Ack";
-		    break;
-		case PS_TS_WACK_ORDREL:
-		    cp = "Internal";
-		    break;
-		default:
-		    (void) snpf(sbuf, sizeof(sbuf), "UNKNOWN_TPI_STATE_%u", u);
+	    case 1:				/* UDP */
+		if (!UdpSt)
+		    (void) build_IPstates();
+		if ((u = Lf->lts.state.ui + UdpStOff) > UdpNstates) {
+		    (void) snpf(sbuf, sizeof(sbuf), "UNKNOWN_TPI_STATE_%u",
+			Lf->lts.state.ui);
 		    cp = sbuf;
+		} else
+		    cp = UdpSt[u];
+	    }
+	    if (cp) {
+		if (Ffield)
+		    (void) printf("%cST=%s%c", LSOF_FID_TCPTPI, cp, Terminator);
+		else {
+		    putchar('(');
+		    (void) fputs(cp, stdout);
 		}
-	    }
-	    if (Ffield)
-		(void) printf("%cST=%s%c", LSOF_FID_TCPTPI, cp, Terminator);
-	    else {
-		putchar('(');
-		(void) fputs(cp, stdout);
-	    }
 	    ps++;
+	    }
 	}
 
 #if	defined(HASTCPTPIQ)
@@ -640,10 +955,11 @@ print_tcptpi(nl)
 void
 process_socket(f, s)
 	struct pst_fileinfo2 *f;		/* file information */
-	struct pst_socket *s;			/* socket information */
+	struct pst_socket *s;			/* optional socket information
+						 * NULL == none */
 {
+	int af, err, fp, lp, tx;
 	char buf[1024], tbuf[32];
-	int err, fp, lp;
 	unsigned char *fa = (unsigned char *)NULL;
 	unsigned char *la = (unsigned char *)NULL;
 	size_t len;
@@ -651,14 +967,112 @@ process_socket(f, s)
 	char *nma = (char *)NULL;
 	struct pst_filedetails pd;
 	struct sockaddr_in *sa;
+	int sx;
 
 #if	defined(HASIPv6)
-	int af;
 	struct sockaddr_in6 *sa6;
 #endif	/* defined(HASIPv6) */
 
 	struct sockaddr_un *ua;
+/*
+ * Read socket info, as required, so that the protocol state names can be
+ * tested as soon as possible.
+ */
+	if (!s) {
+	    if (!(s = read_sock(f))) {
+		(void) snpf(Namech, Namechl,
+		    "can't read pst_socket%s%s", errno ? ": " : "",
+		    errno ? strerror(errno) : "");
+		(void) enter_nm(Namech);
+		return;
+	    }
+	}
+/*
+ * Collect protocol details so the protocol state name might be tested,
+ * as requested by options.
+ */
+	switch (s->pst_family) {
+	case PS_AF_INET:
+	    af = 4;
+	    break;
 
+#if	defined(HASIPv6)
+	case PS_AF_INET6:
+	    af = 6;
+	    break;
+#endif	/* defined(HASIPv6) */
+
+	default:
+	    af = -1;
+	}
+	switch (s->pst_protocol) {
+	case PS_PROTO_TCP:
+	    sx = (int)s->pst_pstate + TcpStOff;
+	    tx = 0;
+	    break;
+	case PS_PROTO_UDP:
+	    sx = (unsigned int)s->pst_pstate + UdpStOff;
+	    tx = 1;
+	    break;
+	default:
+	    sx = tx = -1;
+	}
+/*
+ * Test the protocol state and name, setting the SELNET flag where possible.
+ */
+	switch (tx) {
+	case 0:					/* TCP */
+	    if (TcpStXn) {
+
+	    /*
+	     * Check for TCP state exclusion.
+	     */
+		if (sx >= 0 && sx < TcpNstates) {
+		    if (TcpStX[sx]) {
+			Lf->sf |= SELEXCLF;
+			return;
+		    }
+		}
+	    }
+	    if (TcpStIn) {
+		if (sx >= 0 && sx < TcpNstates) {
+		    if (TcpStI[sx])
+			TcpStI[sx] = 2;
+		    else {
+			Lf->sf |= SELEXCLF;
+			return;
+		    }
+		}
+	    }
+	    break;
+	case 1:					/* UDP */
+	    if (UdpStXn) {
+
+	    /*
+	     * Check for UDP state exclusion.
+	     */
+		if (sx >= 0 && sx < UdpNstates) {
+		    if (UdpStX[sx]) {
+			Lf->sf |= SELEXCLF;
+			return;
+		    }
+		}
+	    }
+	    if (UdpStIn) {
+		if (sx >= 0 && sx < UdpNstates) {
+		    if (UdpStI[sx])
+			UdpStI[sx] = 2;
+		    else {
+			Lf->sf |= SELEXCLF;
+			return;
+		    }
+		}
+	    }
+	    break;
+	}
+/*
+ * Set default type.
+ */
 	(void) snpf(Lf->type, sizeof(Lf->type), "sock");
 	Lf->inp_ty = 2;
 /*
@@ -676,18 +1090,6 @@ process_socket(f, s)
 	}
 #endif	/* defined(HASFSTRUCT) */
 
-/*
- * Read socket info, as required.
- */
-	if (!s) {
-	    if (!(s = read_sock(f))) {
-		(void) snpf(Namech, Namechl,
-		    "can't read pst_socket%s%s", errno ? ": " : "",
-		    errno ? strerror(errno) : "");
-		(void) enter_nm(Namech);
-		return;
-	    }
-	}
 /*
  * Save size information, as requested.
  */
@@ -711,7 +1113,7 @@ process_socket(f, s)
 	case PS_AF_INET6:
 	    Lf->lts.rq = (unsigned long)s->pst_idata;
 	    Lf->lts.sq = (unsigned long)s->pst_odata;
-	    Lf->lts.rqs = Lf->lts.sqs = 1;
+	    Lf->lts.rqs = Lf->lts.sqs = (unsigned char)1;
 	}
 #endif  /* defined(HASTCPTPIQ) */
 
@@ -742,7 +1144,7 @@ process_socket(f, s)
 	case PS_AF_INET6:
 	    Lf->lts.rw = (unsigned long)s->pst_rwnd;
 	    Lf->lts.ww = (unsigned long)s->pst_swnd;
-	    Lf->lts.rws = Lf->lts.wws = 1;
+	    Lf->lts.rws = Lf->lts.wws = (unsigned char)1;
 	}
 #endif	/* defined(HASTCPTPIW) */
 
@@ -751,7 +1153,7 @@ process_socket(f, s)
  */
 	switch (s->pst_family) {
 	case PS_AF_INET:
-	    if (Fnet && (FnetTy != 6))
+	    if (Fnet && (!FnetTy || (FnetTy != 6)))
 		Lf->sf |= SELNET;
 	    (void) snpf(Lf->type, sizeof(Lf->type),
 	    
@@ -771,7 +1173,7 @@ process_socket(f, s)
 		break;
 	    case PS_PROTO_UDP:
 		Lf->lts.type = 1;
-		Lf->lts.state.i = (unsigned int)s->pst_pstate;
+		Lf->lts.state.ui = (unsigned int)s->pst_pstate;
 	    }
 	/*
 	 * Enter local and remote addresses, being careful to generate
@@ -796,7 +1198,7 @@ process_socket(f, s)
 #if	defined(HASIPv6)
 	case PS_AF_INET6:
 	    af = AF_INET6;
-	    if (Fnet && (FnetTy != 4))
+	    if (Fnet && (!FnetTy || (FnetTy != 4)))
 		Lf->sf |= SELNET;
 	    (void) snpf(Lf->type, sizeof(Lf->type), "IPv6");
 	    printpsproto(s->pst_protocol);
@@ -968,19 +1370,33 @@ process_socket(f, s)
  */
 
 void
-process_stream(f)
+process_stream(f, ckscko)
 	struct pst_fileinfo2 *f;		/* pst_fileinfo2 */
+	int ckscko;				/* socket file only checking
+						 * if 1 */
 {
 	struct clone *cl;
 	char *cp;
 	struct l_dev *dp = (struct l_dev *)NULL;
-	int i, ncx, nsn, nsr;
+	int hx, i, ncx, nsn, nsr;
 	size_t nb, nl;
 	KA_T na;
 	static int nsa = 0;
 	dev_t rdev;
 	static struct pst_stream *s = (struct pst_stream *)NULL;
+	struct pst_socket sck;
 	static size_t sz = sizeof(struct pst_stream);
+
+#if	!defined(PS_STR_XPORT_DATA)
+/*
+ * If socket file only checking is enabled and this HP-UX PSTAT instance
+ * doesn't support TCP or UDP stream eXPORT data, return without further
+ * action.
+ */
+	if (ckscko == 1)
+		return;
+#endif	/* !defined(PS_STR_XPORT_DATA) */
+
 /*
  * Generate and save node ID.
  */
@@ -1039,27 +1455,66 @@ process_stream(f)
 	    return;
 	}
 /*
- * Make sure the stream head's fileid and nodeid match the ones in the
- * pst_fileino2 structure.  Enter size from stream head's structure,
- * if requested.
+ * Find the stream head.
  */
-	if ((f->psf_hi_fileid != s[0].val.head.pst_hi_fileid)
-	 |  (f->psf_lo_fileid != s[0].val.head.pst_lo_fileid)
-	 |  (f->psf_hi_nodeid != s[0].val.head.pst_hi_nodeid)
-	 |  (f->psf_lo_nodeid != s[0].val.head.pst_lo_nodeid)) {
+	for (hx = 0; hx < nsn; hx++) {
+	    if (s[hx].type == PS_STR_HEAD)
+		break;
+	}
+	if (hx >= nsn) { 
+	    enter_nm("no stream head located");
+	    return;
+	}
+/*
+ * Make sure the stream head's fileid and nodeid match the ones in the
+ * pst_fileino2 structure.
+ */
+	if ((f->psf_hi_fileid != s[hx].val.head.pst_hi_fileid)
+	 |  (f->psf_lo_fileid != s[hx].val.head.pst_lo_fileid)
+	 |  (f->psf_hi_nodeid != s[hx].val.head.pst_hi_nodeid)
+	 |  (f->psf_lo_nodeid != s[hx].val.head.pst_lo_nodeid)) {
 	    enter_nm("no matching stream data available");
 	    return;
 	}
+
+#if	defined(PS_STR_XPORT_DATA)
+/*
+ * See if this stream has eXPORT data available and is a TCP or
+ * UDP stream.
+ */
+	if ((s[hx].pst_extn_flags & PS_STR_XPORT_DATA)
+	&&  ((s[hx].pst_str_xport_protocol == PS_STR_XPORT_TCP_PROTO)
+	||   (s[hx].pst_str_xport_protocol == PS_STR_XPORT_UDP_PROTO))
+	) {
+
+	/*
+	 * Make a socket from the eXPORT data and process it.
+	 */
+	    (void) make_sock(f, &s[hx], &sck);
+	    (void) process_socket(f, &sck);
+	    return;
+	} else if (ckscko || Selinet) {
+
+	/*
+	 * If socket file or Internet file only processing is enabled, return.
+	 */
+	    return;
+	}
+#endif	/* defined(PS_STR_XPORT_DATA) */
+
+/*
+ * Enter size from stream head's structure, if requested.
+ */
 	if (Fsize) {
 	    if (Lf->access == 'r') {
-		Lf->sz = (SZOFFTYPE)s[0].val.head.pst_rbytes;
+		Lf->sz = (SZOFFTYPE)s[hx].val.head.pst_rbytes;
 		Lf->sz_def = 1;
 	    } else if (Lf->access == 'w') {
-		Lf->sz = (SZOFFTYPE)s[0].val.head.pst_wbytes;
+		Lf->sz = (SZOFFTYPE)s[hx].val.head.pst_wbytes;
 		Lf->sz_def = 1;
 	    } else if (Lf->access == 'u') {
-		Lf->sz = (SZOFFTYPE)s[0].val.head.pst_rbytes
-		       + (SZOFFTYPE)s[0].val.head.pst_wbytes;
+		Lf->sz = (SZOFFTYPE)s[hx].val.head.pst_rbytes
+		       + (SZOFFTYPE)s[hx].val.head.pst_wbytes;
 		Lf->sz_def = 1;
 	    }
 	}
@@ -1081,26 +1536,26 @@ process_stream(f)
  */
 	if (!Sdev)
 	    readdev(0);
-	if (s[0].val.head.pst_flag & PS_STR_ISACLONE) {
-	    if (HaveCloneMaj && (CloneMaj == s[0].val.head.pst_dev_major)) {
+	if (s[hx].val.head.pst_flag & PS_STR_ISACLONE) {
+	    if (HaveCloneMaj && (CloneMaj == s[hx].val.head.pst_dev_major)) {
 		for (cl = Clone; cl; cl = cl->next) {
 		    if (GET_MIN_DEV(Devtp[cl->dx].rdev)
-		    ==  s[0].val.head.pst_dev_minor)
+		    ==  s[hx].val.head.pst_dev_minor)
 		    {
 			dp = &Devtp[cl->dx];
 			break;
 		    }
 		}
 	    } else {
-		rdev = makedev(s[0].val.head.pst_dev_major,
-			       s[0].val.head.pst_dev_minor);
+		rdev = makedev(s[hx].val.head.pst_dev_major,
+			       s[hx].val.head.pst_dev_minor);
 		dp = lkupdev(&DevDev, &rdev, 0, 1);
 	    }
-	    rdev = makedev(s[0].val.head.pst_dev_minor,
-			   s[0].val.head.pst_dev_seq);
+	    rdev = makedev(s[hx].val.head.pst_dev_minor,
+			   s[hx].val.head.pst_dev_seq);
 	} else {
-	    rdev = makedev(s[0].val.head.pst_dev_major,
-			   s[0].val.head.pst_dev_minor);
+	    rdev = makedev(s[hx].val.head.pst_dev_major,
+			   s[hx].val.head.pst_dev_minor);
 	    dp = lkupdev(&DevDev, &rdev, 0, 1);
 	}
 	Lf->dev = DevDev;
