@@ -1,5 +1,5 @@
 /*
- * dnode1.h - SunOS (Solaris 1.x and 2.x) AFS support
+ * dnode1.h - Solaris AFS support
  */
 
 
@@ -33,7 +33,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode1.c,v 1.6 99/05/04 09:03:59 abe Exp $";
+static char *rcsid = "$Id: dnode1.c,v 1.9 2005/08/08 19:55:41 abe Exp $";
 #endif
 
 
@@ -44,14 +44,10 @@ static char *rcsid = "$Id: dnode1.c,v 1.6 99/05/04 09:03:59 abe Exp $";
 #define __XDR_INCLUDE__
 #define int32   old_solaris_int32
 
-# if	!defined(solaris)
-#undef	END
-# endif	/* !defined(solaris) */
-
-# if	defined(solaris) && solaris>=20600
+# if	solaris>=20600
 #undef	SHARED
 #undef	PRIVATE
-# endif	/* defined(solaris) && solaris>=20600 */
+# endif	/* solaris>=20600 */
 
 #include <afs/param.h>
 #include <afs/afsint.h>
@@ -63,40 +59,35 @@ static char *rcsid = "$Id: dnode1.c,v 1.6 99/05/04 09:03:59 abe Exp $";
  * the AFS sources in afs/lock.h.
  */
 
-# if	defined(solaris)
-#  if	defined(AFS_SUN5_ENV)
+# if	defined(AFS_SUN5_ENV)
 #define AFS_NOBOZO_LOCK
-#  endif	/* defined(AFS_SUN5_ENV) */
+# endif	/* defined(AFS_SUN5_ENV) */
 
 #define INSTRUMENT_LOCKS
 
-#  if	defined(AFS_FINEGR_SUNLOCK)
+# if	defined(AFS_FINEGR_SUNLOCK)
 typedef	kmutex_	afs_lock_t;
 typedef	krwlock_t afs_rwlock_t;
-#  endif	/* !defined(AFS_FINEGR_SUNLOCK) */
+# endif	/* !defined(AFS_FINEGR_SUNLOCK) */
 
 struct afs_lock {
 
-#  if	solaris>=20500
+# if	solaris>=20500
     unsigned char d1[2];
     unsigned short d1_5[3];
-#  else	/* solaris < 20500 */
+# else	/* solaris < 20500 */
     unsigned char d1[4];
-#  endif	/* solaris>=20500 */
+# endif	/* solaris>=20500 */
 
     struct timeval d2;
 
-#  if	defined(INSTRUMENT_LOCKS)
+# if	defined(INSTRUMENT_LOCKS)
     unsigned int d3[3];
-#  endif	/* defined(INSTRUMENT_LOCKS) */
+# endif	/* defined(INSTRUMENT_LOCKS) */
 
 };
 typedef struct afs_lock afs_lock_t;
 typedef struct afs_lock afs_rwlock_t;
-# else	/* !defined(solaris) */
-typedef	int32 afs_lock_t;
-typedef	int32 afs_rwlock_t;
-# endif	/* defined(solaris) */
 
 
 /*
@@ -395,9 +386,10 @@ readafsnode(va, v, an)
 	ka = (KA_T)((char *)va + sizeof(struct vnode));
 	len = sizeof(struct vcache) - sizeof(struct vnode);
 	if (kread(ka, cp, len)) {
-	    (void) sprintf(Namech,
+	    (void) snpf(Namech, Namechl,
 		"vnode at %s: can't read vcache remainder from %s",
-		print_kptr(va, tbuf), print_kptr(ka, (char *)NULL));
+		print_kptr(va, tbuf, sizeof(tbuf)),
+		print_kptr(ka, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return(1);
 	}
@@ -413,14 +405,14 @@ readafsnode(va, v, an)
  */
 	if (vc->mvstat == 2) {
 	    if ((vp = getvolume(&vc->fid, &vols))) {
-		an->inode = (unsigned long)((vp->mtpoint.Fid.Vnode
-			  +		    (vp->mtpoint.Fid.Volume << 16))
+		an->inode = (INODETYPE)((vp->mtpoint.Fid.Vnode
+			  +		(vp->mtpoint.Fid.Volume << 16))
 			  & 0x7fffffff);
-		if (an->inode == (unsigned long)0) {
+		if (an->inode == (INODETYPE)0) {
 		    if (is_rootFid(vc, &rfid))
 			an->ino_st = 1;
 		    else if (rfid) {
-			an->inode = (unsigned long)2;
+			an->inode = (INODETYPE)2;
 			an->ino_st = 1;
 		    } else
 			an->ino_st = 0;
@@ -428,18 +420,18 @@ readafsnode(va, v, an)
 		    an->ino_st = 1;
 	    } else {
 		if (vols) {
-		    an->inode = (unsigned long)2;
+		    an->inode = (INODETYPE)2;
 		    an->ino_st = 1;
 		} else {
 		    if (v->v_flag & VROOT) {
-			an->inode = (unsigned long)0;
+			an->inode = (INODETYPE)0;
 			an->ino_st = 1;
 		    } else
 			an->ino_st = 0;
 		}
 	    }
 	} else {
-	    an->inode = (unsigned long)((vc->fid.Fid.Vnode
+	    an->inode = (INODETYPE)((vc->fid.Fid.Vnode
 		      +			(vc->fid.Fid.Volume << 16))
 		      & 0x7fffffff);
 	    an->ino_st = 1;
