@@ -31,7 +31,7 @@
 
 
 /*
- * To use this souce file:
+ * To use this source file:
  *
  * 1. Define USE_LIB_IS_FILE_NAMED.
  *
@@ -61,7 +61,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: isfn.c,v 1.4 99/03/29 07:08:51 abe Exp $";
+static char *rcsid = "$Id: isfn.c,v 1.10 2008/10/21 16:12:36 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -89,6 +89,9 @@ static int HbyCdCt = 0;			/* HbyCd entry count */
 static struct hsfile *HbyFdi =		/* hash by file (dev,ino) buckets */
 	(struct hsfile *)NULL;
 static int HbyFdiCt = 0;		/* HbyFdi entry count */
+static struct hsfile *HbyFrd =		/* hash by file raw device buckets */
+	(struct hsfile *)NULL;
+static int HbyFrdCt = 0;		/* HbyFrd entry count */
 static struct hsfile *HbyFsd =		/* hash by file system buckets */
 	(struct hsfile *)NULL;
 static int HbyFsdCt = 0;		/* HbyFsd entry count */
@@ -112,10 +115,17 @@ static int HbyNmCt = 0;			/* HbyNm entry count */
 					 * number bucket count (power of 2!) */
 #define SFHASHDEVINO(maj, min, ino, mod) ((int)(((int)((((int)(maj+1))*((int)((min+1))))+ino)*31415)&(mod-1)))
 					/* hash for Sfile by major device,
-					 * minor device, and inode, modulo m
-					 * (m must be a power of 2) */
+					 * minor device, and inode, modulo mod
+					 * (mod must be a power of 2) */
+#define	SFRDHASH	1024		/* Sfile hash by raw device number
+					 * bucket count (power of 2!) */
+#define SFHASHRDEVI(maj, min, rmaj, rmin, ino, mod) ((int)(((int)((((int)(maj+1))*((int)((min+1))))+((int)(rmaj+1)*(int)(rmin+1))+ino)*31415)&(mod-1)))
+					/* hash for Sfile by major device,
+					 * minor device, major raw device,
+					 * minor raw device, and inode, modulo
+					 * mod (mod must be a power of 2) */
 #define	SFNMHASH	4096		/* Sfile hash by name bucket count
-					   (power of 2!) */
+					 * (must be a power of 2!) */
 
 
 
@@ -128,7 +138,7 @@ hashSfile()
 {
 	static int hs = 0;
 	int i;
-	int sfplm = 2;
+	int sfplm = 3;
 	struct sfile *s;
 	struct hsfile *sh, *sn;
 /*
@@ -151,7 +161,7 @@ hashSfile()
 		    Pn, SFCDHASH);
 		Exit(1);
 	    }
-	    sfplm = 3;
+	    sfplm++;
 	}
 # endif	/* defined(HAVECLONEMAJ) */
 
@@ -163,6 +173,14 @@ hashSfile()
 		Pn, SFDIHASH);
 	    Exit(1);
 	}
+	if (!(HbyFrd = (struct hsfile *)calloc((MALLOC_S)SFRDHASH,
+					       sizeof(struct hsfile))))
+	{
+	    (void) fprintf(stderr,
+		"%s: can't allocate space for %d rdev hash buckets\n",
+		Pn, SFRDHASH);
+	    Exit(1);
+	}
 	if (!(HbyFsd = (struct hsfile *)calloc((MALLOC_S)SFFSHASH,
 					       sizeof(struct hsfile))))
 	{
@@ -181,8 +199,8 @@ hashSfile()
 	}
 	hs++;
 /*
- * Scan the Sfile chain, building file, file system, and file name hash
- * bucket chains.
+ * Scan the Sfile chain, building file, file system, raw device, and file
+ * name hash bucket chains.
  */
 	for (s = Sfile; s; s = s->next) {
 	    for (i = 0; i < sfplm; i++) {
@@ -193,24 +211,36 @@ hashSfile()
 		    HbyNmCt++;
 		} else if (i == 1) {
 		    if (s->type) {
-			sh = &HbyFdi[SFHASHDEVINO(major(s->dev),
-						  minor(s->dev), s->i,
+			sh = &HbyFdi[SFHASHDEVINO(GET_MAJ_DEV(s->dev),
+						  GET_MIN_DEV(s->dev), s->i,
 						  SFDIHASH)];
 			HbyFdiCt++;
 		    } else {
-			sh = &HbyFsd[SFHASHDEVINO(major(s->dev),
-						  minor(s->dev),
+			sh = &HbyFsd[SFHASHDEVINO(GET_MAJ_DEV(s->dev),
+						  GET_MIN_DEV(s->dev),
 						  0,
 						  SFFSHASH)];
 			HbyFsdCt++;
 		    }
+		} else if (i == 2) {
+		    if ((s->mode == S_IFCHR) || (s->mode == S_IFBLK)) {
+			sh = &HbyFrd[SFHASHRDEVI(GET_MAJ_DEV(s->dev),
+						 GET_MIN_DEV(s->dev),
+						 GET_MAJ_DEV(s->rdev),
+						 GET_MIN_DEV(s->rdev),
+						 s->i,
+						 SFRDHASH)];
+			HbyFrdCt++;
+		    } else
+			continue;
 		}
 
 # if	defined(HAVECLONEMAJ)
 		else {
-		    if (!HAVECLONEMAJ || (major(s->dev) != CLONEMAJ))
+		    if (!HAVECLONEMAJ || (GET_MAJ_DEV(s->rdev) != CLONEMAJ))
 			continue;
-		    sh = &HbyCd[SFHASHDEVINO(0, minor(s->dev), 0, SFCDHASH)];
+		    sh = &HbyCd[SFHASHDEVINO(0, GET_MIN_DEV(s->rdev), 0,
+					     SFCDHASH)];
 		    HbyCdCt++;
 		}
 # else	/* ! defined(HAVECLONEMAJ) */
@@ -248,13 +278,15 @@ int
 is_file_named(p, cd)
 	char *p;			/* path name; NULL = search by device
 					 * and inode (from *Lf) */
-	int cd;				/* character-type file -- e.g.,
-					 * VCHR vnode or S_IFCHR inode */
+	int cd;				/* character or block type file --
+					 * VCHR or VBLK vnode, or S_IFCHR
+					 * or S_IFBLK inode */
 {
+	char *ep;
 	int f = 0;
-	int h;
-	struct sfile *s;
+	struct sfile *s = (struct sfile *)NULL;
 	struct hsfile *sh;
+	size_t sz;
 /*
  * Check for a path name match, as requested.
  */
@@ -271,12 +303,16 @@ is_file_named(p, cd)
 /*
  * If this is a stream, check for a clone device match.
  */
-	if (!f && HbyCdCt && Lf->is_stream && Lf->dev_def) {
-	    for (sh = &HbyCd[SFHASHDEVINO(0, major(Lf->dev), 0, SFCDHASH)];
+	if (!f && HbyCdCt && Lf->is_stream && Lf->dev_def && Lf->rdev_def
+	&&  (Lf->dev == DevDev))
+	{
+	    for (sh = &HbyCd[SFHASHDEVINO(0, GET_MAJ_DEV(Lf->rdev), 0,
+					  SFCDHASH)];
 		 sh;
 		 sh = sh->next)
 	    {
-		if ((s = sh->s) && (major(Lf->dev) == minor(s->dev))) {
+		if ((s = sh->s) && (GET_MAJ_DEV(Lf->rdev)
+		==		    GET_MIN_DEV(s->rdev))) {
 		    f = 3;
 		    break;
 		}
@@ -290,34 +326,57 @@ is_file_named(p, cd)
 	if (!f && HbyFdiCt && Lf->dev_def
 	&& (Lf->inp_ty == 1 || Lf->inp_ty == 3))
 	{
-	    for (sh = &HbyFdi[SFHASHDEVINO(major(Lf->dev),
-					   minor(Lf->dev),
+	    for (sh = &HbyFdi[SFHASHDEVINO(GET_MAJ_DEV(Lf->dev),
+					   GET_MIN_DEV(Lf->dev),
 					   Lf->inode,
 					   SFDIHASH)];
 		 sh;
 		 sh = sh->next)
 	    {
 		if ((s = sh->s) && (Lf->dev == s->dev)
-		&&  ((ino_t)Lf->inode == s->i)) {
+		&&  (Lf->inode == s->i)) {
 		    f = 1;
 		    break;
 		}
 	    }
 	}
 /*
- * Check for a file system.
+ * Check for a file system match.
  */
 	if (!f && HbyFsdCt && Lf->dev_def) {
-	    for (sh = &HbyFsd[SFHASHDEVINO(major(Lf->dev), minor(Lf->dev), 0,
+	    for (sh = &HbyFsd[SFHASHDEVINO(GET_MAJ_DEV(Lf->dev),
+	    				   GET_MIN_DEV(Lf->dev), 0,
 					   SFFSHASH)];
 		 sh;
 		 sh = sh->next)
 	    {
-		if ((s = sh->s) && Lf->dev == s->dev) {
-		    if (!cd || s->mode == S_IFCHR) {
-			f = 1;
-			break;
-		    }
+		if ((s = sh->s) && (s->dev == Lf->dev)) {
+		    f = 1;
+		    break;
+		}
+	    }
+	}
+/*
+ * Check for a character or block device match.
+ */
+	if (!f && HbyFrdCt && cd
+	&&  Lf->dev_def && (Lf->dev == DevDev)
+	&&  Lf->rdev_def
+	&& (Lf->inp_ty == 1 || Lf->inp_ty == 3))
+	{
+	    for (sh = &HbyFrd[SFHASHRDEVI(GET_MAJ_DEV(Lf->dev),
+					  GET_MIN_DEV(Lf->dev),
+					  GET_MAJ_DEV(Lf->rdev),
+					  GET_MIN_DEV(Lf->rdev),
+					  Lf->inode, SFRDHASH)];
+		 sh;
+		 sh = sh->next)
+	    {
+		if ((s = sh->s) && (s->dev == Lf->dev)
+		&&  (s->rdev == Lf->rdev) && (s->i == Lf->inode))
+		{
+		    f = 1;
+		    break;
 		}
 	    }
 	}
@@ -334,9 +393,11 @@ is_file_named(p, cd)
 	     * If the search argument isn't a file system, propagate it
 	     * to Namech[]; otherwise, let printname() compose the name.
 	     */
-		(void) strcpy(Namech, s->name);
-		if (s->devnm)
-		    (void) sprintf(endnm(), " (%s)", s->devnm);
+		(void) snpf(Namech, Namechl, "%s", s->name);
+		if (s->devnm) {
+		    ep = endnm(&sz);
+		    (void) snpf(ep, sz, " (%s)", s->devnm);
+		}
 	    }
 	    break;
 	case 2:
@@ -348,9 +409,10 @@ is_file_named(p, cd)
 # endif	/* defined(HAVECLONEMAJ) */
 
 	}
-	s->f = 1;
+	if (s)
+	    s->f = 1;
 	return(1);
 }
 #else	/* !defined(USE_LIB_IS_FILE_NAMED) */
-static char d1[] = "d"; static char *d2 = d1;
+char isfn_d1[] = "d"; char *isfn_d2 = isfn_d1;
 #endif	/* defined(USE_LIB_IS_FILE_NAMED) */
