@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.48 2005/05/11 13:02:46 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.57 2015/07/07 20:16:58 abe Exp $";
 #endif
 
 
@@ -65,11 +65,12 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	int c, i, n, rv;
+	int ad, c, i, n, rv, se1, se2, ss;
 	char *cp;
 	int err = 0;
 	int ev = 0;
 	int fh = 0;
+	char *fmtr = (char *)NULL;
 	long l;
 	MALLOC_S len;
 	struct lfile *lf;
@@ -80,14 +81,29 @@ main(argc, argv)
 	struct sfile *sfp;
 	struct lproc **slp = (struct lproc **)NULL;
 	int sp = 0;
-	struct str_lst *str;
+	struct str_lst *str, *strt;
 	int version = 0;
 	int xover = 0;
 
+#if	defined(HAS_STRFTIME)
+	char *fmt = (char *)NULL;
+	size_t fmtl = (size_t)0;
+#endif	/* defined(HAS_STRFTIME) */
+
 #if	defined(HASZONES)
 	znhash_t *zp;
 #endif	/* defined(HASZONES) */
 
+#if	defined(HASSELINUX)
+/*
+ * This stanza must be immediately before the "Save progam name." code, since
+ * it contains code itself.
+ */
+	cntxlist_t *cntxp;
+
+	CntxStatus = is_selinux_enabled() ? 1 : 0;
+#endif	/* defined(HASSELINUX) */
+
 /*
  * Save program name.
  */
@@ -96,14 +112,17 @@ main(argc, argv)
 	else
 	    Pn = argv[0];
 /*
- * Close all file descriptors above 2.
+ * Close enough file descriptors above 2 that library functions will have
+ * open descriptors.
  *
  * Make sure stderr, stdout, and stdin are open descriptors.  Open /dev/null
  * for ones that aren't.  Be terse.
  *
  * Make sure umask allows lsof to define its own file permissions.
  */
-	for (i = 3, n = GET_MAX_FD(); i < n; i++)
+	if ((MaxFd = (int) GET_MAX_FD()) < 53)
+	    MaxFd = 53;
+	for (i = 3; i < MaxFd; i++)
 	    (void) close(i);
 	while (((i = open("/dev/null", O_RDWR, 0)) >= 0) && (i < 2))
 	    ;
@@ -138,7 +157,7 @@ main(argc, argv)
  * Create option mask.
  */
 	(void) snpf(options, sizeof(options),
-	    "?a%sbc:D:d:%sf:F:g:hi:%slL:%sMnNo:Op:Pr:%ssS:tT:u:UvVwx:%s%s",
+	    "?a%sbc:%sD:d:%s%sf:F:g:hi:%s%slL:%s%snNo:Op:Pr:%ss:S:tT:u:UvVwx:%s%s%s",
 
 #if	defined(HAS_AFS) && defined(HASAOPT)
 	    "A:",
@@ -152,18 +171,42 @@ main(argc, argv)
 	    "",
 #endif	/* defined(HASNCACHE) */
 
+#if	defined(HASEOPT)
+	    "e:",
+#else	/* !defined(HASEOPT) */
+	    "",
+#endif	/* defined(HASEOPT) */
+
+#if	defined(HASEPTOPTS)
+	    "E",
+#else	/* !defined(HASEPTOPTS) */
+	    "",
+#endif	/* defined(HASEPTOPTS) */
+
 #if	defined(HASKOPT)
 	    "k:",
 #else	/* !defined(HASKOPT) */
 	    "",
 #endif	/* defined(HASKOPT) */
 
+#if	defined(HASTASKS)
+	    "K",
+#else	/* !defined(HASTASKS) */
+	    "",
+#endif	/* defined(HASTASKS) */
+
 #if	defined(HASMOPT) || defined(HASMNTSUP)
 	    "m:",
 #else	/* !defined(HASMOPT) && !defined(HASMNTSUP) */
 	    "",
 #endif	/* defined(HASMOPT) || defined(HASMNTSUP) */
 
+#if	defined(HASNORPC_H)
+	    "",
+#else	/* !defined(HASNORPC_H) */
+	    "M",
+#endif	/* defined(HASNORPC_H) */
+
 #if	defined(HASPPID)
 	    "R",
 #else	/* !defined(HASPPID) */
@@ -181,10 +224,16 @@ main(argc, argv)
 #endif	/* defined(HASXOPT) */
 
 #if	defined(HASZONES)
-	    "z:"
+	    "z:",
 #else	/* !defined(HASZONES) */
-	    ""
+	    "",
 #endif	/* defined(HASZONES) */
+ 
+#if	defined(HASSELINUX)
+	    "Z:"
+#else	/* !defined(HASSELINUX) */
+	    ""
+#endif	/* defined(HASSELINUX) */
 
 	    );
 /*
@@ -195,7 +244,7 @@ main(argc, argv)
 		err = 1;
 		continue;
 	    }
-	    switch(c) {
+	    switch (c) {
 	    case 'a':
 		Fand = 1;
 		break;
@@ -229,16 +278,41 @@ main(argc, argv)
 			    GOx1 = GObk[0];
 			    GOx2 = GObk[1];
 			}
-		    } else
+		    } else {
 			CmdLim = atoi(GOv);
+
+#if	defined(MAXSYSCMDL)
+			if (CmdLim > MAXSYSCMDL) {
+			    (void) fprintf(stderr,
+				"%s: +c %d > what system provides (%d)\n",
+				Pn, CmdLim, MAXSYSCMDL);
+			    err = 1;
+			}
+#endif	/* defined(MAXSYSCMDL) */
+
+		    }
 		    break;
 		}
 		if (GOv && (*GOv == '/')) {
 		    if (enter_cmd_rx(GOv))
 			err = 1;
 		} else {
-		    if (enter_str_lst("-c", GOv, &Cmdl))
+		    if (enter_str_lst("-c", GOv, &Cmdl, &Cmdni, &Cmdnx))
 			err = 1;
+
+#if	defined(MAXSYSCMDL)
+		    else if (Cmdl->len > MAXSYSCMDL) {
+			(void) fprintf(stderr, "%s: \"-c ", Pn);
+			(void) safestrprt(Cmdl->str, stderr, 2);
+			(void) fprintf(stderr, "\" length (%d) > what system",
+			    Cmdl->len);
+			(void) fprintf(stderr, " provides (%d)\n",
+			    MAXSYSCMDL);
+			Cmdl->len = 0;	/* (to avoid later error report) */
+			err = 1;
+		    }
+#endif	/* defined(MAXSYSCMDL) */
+
 		}
 		break;
 
@@ -246,8 +320,8 @@ main(argc, argv)
 	    case 'C':
 		Fncache = (GOp == '-') ? 0 : 1;
 		break;
-#endif	/* defined(HASNCACHE) */
 
+#endif	/* defined(HASNCACHE) */
 	    case 'd':
 		if (GOp == '+') {
 		    if (enter_dir(GOv, 0))
@@ -281,6 +355,20 @@ main(argc, argv)
 
 		}
 		break;
+
+#if	defined(HASEOPT)
+	    case 'e':
+		if (enter_efsys(GOv, ((GOp == '+') ? 1 : 0)))
+		    err = 1;
+		break;
+#endif	/* defined(HASEOPT) */
+
+#if	defined(HASEPTOPTS)
+	    case 'E':
+		FeptE = (GOp == '+') ? 2 : 1;
+		break;
+#endif	/* defined(HASEPTOPTS) */
+
 	    case 'f':
 		if (!GOv || *GOv == '-' || *GOv == '+') {
 		    Ffilesys = (GOp == '+') ? 2 : 1;
@@ -294,35 +382,52 @@ main(argc, argv)
 #if	defined(HASFSTRUCT)
 		for (; *GOv; GOv++) {
 		    switch (*GOv) {
+
+# if	!defined(HASNOFSCOUNT)
 		    case 'c':
 		    case 'C':
-			if (GOp == '+')
+			if (GOp == '+') {
 			    Fsv |= FSV_CT;
-			else
+			    FsvByf = 1;
+			} else
 			    Fsv &= (unsigned char)~FSV_CT;
 			break;
+# endif	/* !defined(HASNOFSCOUNT) */
+
+# if	!defined(HASNOFSADDR)
 		    case 'f':
 		    case 'F':
-			if (GOp == '+')
+			if (GOp == '+') {
 			    Fsv |= FSV_FA;
-			else
+			    FsvByf = 1;
+			} else
 			    Fsv &= (unsigned char)~FSV_FA;
 			break;
+# endif	/* !defined(HASNOFSADDR) */
+
+# if	!defined(HASNOFSFLAGS)
 		    case 'g':
 		    case 'G':
-			if (GOp == '+')
+			if (GOp == '+') {
 			    Fsv |= FSV_FG;
-			else
+			    FsvByf = 1;
+			} else
 			    Fsv &= (unsigned char)~FSV_FG;
 			FsvFlagX = (*GOv == 'G') ? 1 : 0;
 			break;
+# endif	/* !defined(HASNOFSFLAGS) */
+
+# if	!defined(HASNOFSNADDR)
 		    case 'n':
 		    case 'N':
-			if (GOp == '+')
+			if (GOp == '+') {
 			    Fsv |= FSV_NI;
-			else
+			    FsvByf = 1;
+			} else
 			    Fsv &= (unsigned char)~FSV_NI;
 			break;
+# endif	/* !defined(HASNOFSNADDR */
+
 		    default:
 			(void) fprintf(stderr,
 			    "%s: unknown file struct option: %c\n", Pn, *GOv);
@@ -360,14 +465,28 @@ main(argc, argv)
 			||  FieldSel[i].id == LSOF_FID_NI)
 			    continue;
 #endif	/* !defined(HASFSTRUCT) */
+ 
+#if	defined(HASSELINUX)
+			if ((FieldSel[i].id == LSOF_FID_CNTX) && !CntxStatus)
+			    continue;
+#else	/* !defined(HASSELINUX) */
+			if (FieldSel[i].id == LSOF_FID_CNTX)
+			    continue;
+#endif	/* !defined(HASSELINUX) */
+
+			if (FieldSel[i].id == LSOF_FID_RDEV)
+			    continue;	/* for compatibility */
+
+#if	!defined(HASTASKS)
+			if (FieldSel[i].id == LSOF_FID_TID)
+			    continue;
+#endif	/* !defined(HASTASKS) */
 
 #if	!defined(HASZONES)
 			if (FieldSel[i].id == LSOF_FID_ZONE)
 			    continue;
 #endif	/* !defined(HASZONES) */
 
-			if (FieldSel[i].id == LSOF_FID_RDEV)
-			    continue;	/* for compatibility */
 			FieldSel[i].st = 1;
 			if (FieldSel[i].opt && FieldSel[i].ov)
 			    *(FieldSel[i].opt) |= FieldSel[i].ov;
@@ -401,6 +520,11 @@ main(argc, argv)
 			    continue;
 #endif	/* !defined(HASFSTRUCT) */
 
+#if	!defined(HASTASKS)
+			if (FieldSel[i].id == LSOF_FID_TID)
+			    continue;
+#endif	/* !defined(HASTASKS) */
+
 			if (FieldSel[i].id == *GOv) {
 			    FieldSel[i].st = 1;
 			    if (FieldSel[i].opt && FieldSel[i].ov)
@@ -466,6 +590,13 @@ main(argc, argv)
 		break;
 #endif	/* defined(HASKOPT) */
 
+#if	defined(HASTASKS)
+		case 'K':
+		    Ftask = 1;
+		    Selflags |= SELTASK;
+		    break;
+#endif	/* defined(HASTASKS) */
+
 	    case 'l':
 		Futol = 0;
 		break;
@@ -547,9 +678,12 @@ main(argc, argv)
 		break;
 #endif	/* defined(HASMOPT) || defined(HASMNTSUP) */
 
+#if	!defined(HASNORPC_H)
 	    case 'M':
 		FportMap = (GOp == '+') ? 1 : 0;
 		break;
+#endif	/* !defined(HASNORPC_H) */
+
 	    case 'n':
 		Fhost = (GOp == '-') ? 0 : 1;
 		break;
@@ -611,10 +745,48 @@ main(argc, argv)
 		    RptTm = i;
 		else
 		    RptTm = RPTTM;
-		if (*cp) {
+		if (!*cp)
+		     break;
+		while(*cp && (*cp == ' '))
+		    cp++;
+		if (*cp != LSOF_FID_MARK) {
 		    GOx1 = GObk[0];
 		    GOx2 = GObk[1] + n;
+		    break;
 		}
+
+#if	defined(HAS_STRFTIME)
+
+	    /*
+	     * Collect the strftime(3) format and test it.
+	     */
+		cp++;
+		if ((fmtl = strlen(cp) + 1) < 1) {
+		    (void) fprintf(stderr, "%s: <fmt> too short: \"%s\"\n",
+			Pn, cp);
+		    err = 1;
+		} else {
+		    fmt = cp;
+		    fmtl = (fmtl * 8) + 1;
+		    if (!(fmtr = (char *)malloc((MALLOC_S)fmtl))) {
+			(void) fprintf(stderr,
+			    "%s: no space (%d) for <fmt> result: \"%s\"\n",
+			    Pn, (int)fmtl, cp);
+			    Exit(1);
+		    }
+		    if (util_strftime(fmtr, fmtl - 1, fmt) < 1) {
+			(void) fprintf(stderr, "%s: illegal <fmt>: \"%s\"\n",
+			    Pn, fmt);
+			err = 1;
+		    }
+		}
+
+#else	/* !defined(HAS_STRFTIME) */
+		(void) fprintf(stderr, "%s: m<fmt> not supported: \"%s\"\n",
+		    Pn, cp);
+		err = 1;
+#endif	/* defined(HAS_STRFTIME) */
+
 		break;
 
 #if	defined(HASPPID)
@@ -624,7 +796,22 @@ main(argc, argv)
 #endif	/* defined(HASPPID) */
 
 	    case 's':
+
+#if	defined(HASTCPUDPSTATE)
+		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    Fsize = 1;
+		    if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
+		} else {
+		    if (enter_state_spec(GOv))
+			err = 1;
+		}
+#else	/* !defined(HASTCPUDPSTATE) */
 		Fsize = 1;
+#endif	/* defined(HASTCPUDPSTATE) */
+
 		break;
 	    case 'S':
 		if (!GOv || *GOv == '-' || *GOv == '+') {
@@ -766,12 +953,89 @@ main(argc, argv)
 		}
 		break;
 #endif	/* defined(HASZONES) */
+ 
+#if	defined(HASSELINUX)
+	    case 'Z':
+		if (!CntxStatus) {
+		   (void) fprintf(stderr, "%s: -Z limited to SELinux\n", Pn);
+		    err = 1;
+		} else {
+		    Fcntx = 1;
+		    if (GOv && (*GOv != '-') && (*GOv != '+')) {
+
+		    /*
+		     * Add to the context name argument hash.
+		     */
+			if (enter_cntx_arg(GOv))
+			    err = 1;
+		    } else if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
+		}
+		break;
+#endif	/* defined(HASSELINUX) */
 
 	    default:
 		(void) fprintf(stderr, "%s: unknown option (%c)\n", Pn, c);
 		err = 1;
 	    }
 	}
+/*
+ * Check for argument consistency.
+ */
+	if (Cmdnx && Cmdni) {
+
+	/*
+	 * Check for command inclusion/exclusion conflicts.
+	 */
+	    for (str = Cmdl; str; str = str->next) {
+		if (str->x) {
+		    for (strt = Cmdl; strt; strt = strt->next) {
+			if (!strt->x) {
+			    if (!strcmp(str->str, strt->str)) {
+				(void) fprintf(stderr,
+				    "%s: -c^%s and -c%s conflict.\n",
+				    Pn, str->str, strt->str);
+				err++;
+			    }
+			}
+		    }
+		}
+	    }
+	}
+
+#if	defined(HASTCPUDPSTATE)
+	if (TcpStXn && TcpStIn) {
+
+	/*
+	 * Check for excluded and included TCP states.
+	 */
+	    for (i = 0; i < TcpNstates; i++) {
+		if (TcpStX[i] && TcpStI[i]) {
+		    (void) fprintf(stderr,
+			"%s: can't include and exclude TCP state: %s\n",
+			Pn, TcpSt[i]);
+		    err = 1;
+		}
+	    }
+	}
+	if (UdpStXn && UdpStIn) {
+
+	/*
+	 * Check for excluded and included UDP states.
+	 */
+	    for (i = 0; i < UdpNstates; i++) {
+		if (UdpStX[i] && UdpStI[i]) {
+		    (void) fprintf(stderr,
+			"%s: can't include and exclude UDP state: %s\n",
+			Pn, UdpSt[i]);
+		    err = 1;
+		}
+	    }
+	}
+#endif	/* defined(HASTCPUDPSTATE) */
+
 	if (Fsize && Foffset) {
 	    (void) fprintf(stderr, "%s: -o and -s are mutually exclusive\n",
 		Pn);
@@ -780,15 +1044,66 @@ main(argc, argv)
 	if (Ffield) {
 	    if (Fterse) {
 		(void) fprintf(stderr,
-		    "%s: -f and -t are mutually exclusive\n", Pn);
+		    "%s: -F and -t are mutually exclusive\n", Pn);
 		err++;
 	    }
 	    FieldSel[LSOF_FIX_PID].st = 1;
+
+#if	defined(HAS_STRFTIME)
+	    if (fmtr) {
+
+	    /*
+	     * The field output marker format can't contain "%n" new line
+	     * requests.
+	     */
+		for (cp = strchr(fmt, '%'); cp; cp = strchr(cp, '%')) {
+		    if (*++cp  == 'n') {
+			(void) fprintf(stderr,
+			    "%s: %%n illegal in -r m<fmt> when -F has", Pn);
+			(void) fprintf(stderr,
+			    " been specified: \"%s\"\n", fmt);
+			err++;
+			break;
+		    } else if (*cp == '%')
+			cp++;
+		}
+	    }
+#endif	/* defined(HAS_STRFTIME) */
+
 	}
 	if (Fxover && !xover) {
 	    (void) fprintf(stderr, "%s: -x must accompany +d or +D\n", Pn);
 	    err++;
 	}
+
+#if	defined(HASEOPT)
+	if (Efsysl) {
+
+	/*
+	 * If there are file systems specified by -e options, check them.
+	 */
+	    efsys_list_t *ep;		/* Efsysl pointer */
+	    struct mounts *mp, *mpw;	/* local mount table pointers */
+
+	    if ((mp = readmnt())) {
+		for (ep = Efsysl; ep; ep = ep->next) {
+		    for (mpw = mp; mpw; mpw = mpw->next) {
+			if (!strcmp(mpw->dir, ep->path)) {
+			    ep->mp = mpw;
+			    break;
+			}
+		    }
+		    if (!ep->mp) {
+			(void) fprintf(stderr,
+			    "%s: \"-e %s\" is not a mounted file system.\n",
+			    Pn, ep->path);
+			err++;
+		    }
+		}
+	    }
+	}
+#endif	/* defined(HASEOPT) */
+
 	if (DChelp || err || Fhelp || fh || version)
 	    usage(err ? 1 : 0, fh, version);
 /*
@@ -806,8 +1121,14 @@ main(argc, argv)
 /*
  * Compute the selection flags.
  */
-	if (Cmdl || CmdRx)
+	if ((Cmdl && Cmdni) || CmdRx)
 	    Selflags |= SELCMD;
+ 
+#if	defined(HASSELINUX)
+	if (CntxArg)
+	    Selflags |= SELCNTX;
+#endif	/* defined(HASSELINUX) */
+
 	if (Fdl)
 	    Selflags |= SELFD;
 	if (Fnet)
@@ -849,9 +1170,25 @@ main(argc, argv)
  * Get the device for DEVDEV_PATH.
  */
 	if (stat(DEVDEV_PATH, &sb)) {
-	    (void) fprintf(stderr, "%s: can't stat(%s): %s\n", Pn,
-		DEVDEV_PATH, strerror(errno));
-	    Exit(1);
+	    se1 = errno;
+ 	    if ((ad = strcmp(DEVDEV_PATH, "/dev"))) {
+		if ((ss = stat("/dev", &sb)))
+		    se2 = errno;
+		else
+		    se2 = 0;
+	    } else {
+		se2 = 0;
+		ss = 1;
+	    }
+	    if (ss) {
+		(void) fprintf(stderr, "%s: can't stat(%s): %s\n", Pn,
+		    DEVDEV_PATH, strerror(se1));
+		if (ad) {
+		    (void) fprintf(stderr, "%s: can't stat(/dev): %s\n", Pn,
+		    strerror(se2));
+		}
+		Exit(1);
+	    }
 	}
 	DevDev = sb.st_dev;
 /*
@@ -888,6 +1225,10 @@ main(argc, argv)
 /*
  * Define the size and offset print formats.
  */
+	(void) snpf(options, sizeof(options), "%%%su", INODEPSPEC);
+	InodeFmt_d = sv_fmt_str(options);
+	(void) snpf(options, sizeof(options), "%%#%sx", INODEPSPEC);
+	InodeFmt_x = sv_fmt_str(options);
 	(void) snpf(options, sizeof(options), "0t%%%su", SZOFFPSPEC);
 	SzOffFmt_0t = sv_fmt_str(options);
 	(void) snpf(options, sizeof(options), "%%%su", SZOFFPSPEC);
@@ -950,6 +1291,57 @@ main(argc, argv)
 		NcacheReload = 1;
 #endif	/* defined(HASNCACHE) */
 
+#if	defined(HASEPTOPTS)
+	    /*
+	     * If endpoint info has been requested, make sure it is coded for
+	     * printing.
+	     *
+	     * Lf contents must be preserved, since they may point to a
+	     * malloc()'d area, and since Lf is used throughout the print
+	     */
+		if (FeptE) {
+		    lf = Lf;
+
+		/*
+		 * Check the files that have been selected for printing by
+		 * by some selection criterion other than being a pipe.
+		 */
+		    for (i = 0; i < Nlproc; i++) {
+			Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
+			if (Lp->pss && (Lp->ept & EPT_PIPE))
+			    (void) process_pinfo(0);
+		    }
+		/*
+		 * In a second pass, process unselected endpoint files,
+		 * possibly selecting them for printing.
+		 */
+		    for (i = 0; i < Nlproc; i++) {
+			Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
+			if (Lp->ept & EPT_PIPE_END)
+			    (void) process_pinfo(1);
+		    }
+
+# if	defined(HASUXSOCKEPT)
+		/*
+		 * Process UNIX socket endpoint files in a similar fashion.
+		 */
+		    for (i = 0; i < Nlproc; i++) {
+			Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
+			if (Lp->pss && (Lp->ept & EPT_UXS))
+			    (void) process_uxsinfo(0);
+		    }
+		    for (i = 0; i < Nlproc; i++) {
+			Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
+			if (Lp->ept & EPT_UXS_END) {
+			    (void) process_uxsinfo(1);
+			}
+		    }
+# endif	/* defined(HASUXSOCKEPT) */
+
+		    Lf = lf;
+		}
+#endif	/* defined(HASEPTOPTS) */
+
 	    /*
 	     * Print the selected processes and count them.
 	     *
@@ -976,17 +1368,51 @@ main(argc, argv)
 	 * If conditional repeat mode is in effect, see if it's time to exit.
 	 */
 	    if (RptTm) {
+
+#if	defined(HASEPTOPTS)
+		(void) clear_pinfo();
+#endif	/* defined(HASEPTOPTS) */
+
 		if (rc) {
 		    if (!n)
 			break;
 		    else
 			ev = 0;
 		}
+
+#if	defined(HAS_STRFTIME)
+		if (fmt && fmtr) {
+
+		/*
+		 * Format the marker line.
+		 */
+		    (void) util_strftime(fmtr, fmtl - 1, fmt);
+		    fmtr[fmtl - 1] = '\0';
+		}
+#endif	/* defined(HAS_STRFTIME) */
+
 		if (Ffield) {
 		    putchar(LSOF_FID_MARK);
-		    putchar('\n');
-		} else
-		    puts("=======");
+
+#if	defined(HAS_STRFTIME)
+		    if (fmtr)
+	    	        (void) printf("%s", fmtr);
+#endif	/* defined(HAS_STRFTIME) */
+
+		    putchar(Terminator);
+		    if (Terminator != '\n')
+			putchar('\n');
+		} else {
+
+#if	defined(HAS_STRFTIME)
+		    if (fmtr)
+			cp = fmtr;
+		    else
+#endif	/* defined(HAS_STRFTIME) */
+
+			cp = "=======";
+		    puts(cp);
+		}
 		(void) fflush(stdout);
 		(void) childx();
 		(void) sleep(RptTm);
@@ -1118,6 +1544,38 @@ main(argc, argv)
 	    if (Fverbose)
 		(void) printf("%s: no Internet files located\n", Pn);
 	}
+
+#if	defined(HASTCPUDPSTATE)
+	if (TcpStIn) {
+
+	/*
+	 * Check for included TCP states not located.
+	 */
+	    for (i = 0; i < TcpNstates; i++) {
+		if (TcpStI[i] == 1) {
+		    rv = 1;
+		    if (Fverbose)
+			(void) printf("%s: TCP state not located: %s\n",
+			    Pn, TcpSt[i]);
+		}
+	    }
+	}
+	if (UdpStIn) {
+
+	/*
+	 * Check for included UDP states not located.
+	 */
+	    for (i = 0; i < UdpNstates; i++) {
+		if (UdpStI[i] == 1) {
+		    rv = 1;
+		    if (Fverbose)
+			(void) printf("%s: UDP state not located: %s\n",
+			    Pn, UdpSt[i]);
+		}
+	    }
+	}
+#endif	/* defined(HASTCPUDPSTATE) */
+
 	if (Fnfs && Fnfs < 2) {
 
 	/*
@@ -1140,6 +1598,18 @@ main(argc, argv)
 		    Pn, Spid[i].i);
 	}
 
+#if	defined(HASTASKS)
+	if (Ftask && Ftask < 2) {
+
+	/*
+	 * Report no tasks located.
+	 */
+	    rv = 1;
+	    if (Fverbose)
+		(void) printf("%s: no tasks located\n", Pn);
+	}
+#endif	/* defined(HASTASKS) */
+
 #if	defined(HASZONES)
 	if (ZoneArg) {
 
@@ -1159,6 +1629,24 @@ main(argc, argv)
 	    }
 	}
 #endif	/* defined(HASZONES) */
+ 
+#if	defined(HASSELINUX)
+	if (CntxArg) {
+
+	/*
+	 * Check context argument results.
+	 */
+	    for (cntxp = CntxArg; cntxp; cntxp = cntxp->next) {
+		if (!cntxp->f) {
+		    rv = 1;
+		    if (Fverbose) {
+			(void) printf("%s: context not located: ", Pn);
+			safestrprt(cntxp->cntx, stdout, 1);
+		    }
+		}
+	    }
+	}
+#endif	/* defined(HASSELINUX) */
 
 	for (i = 0; i < Npgid; i++) {
 
