@@ -32,15 +32,20 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: misc.c,v 1.23 2004/10/17 21:39:23 abe Exp $";
+static char *rcsid = "$Id: misc.c,v 1.28 2014/10/13 22:36:20 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
-#if	defined(HASWIDECHAR) && defined(WIDECHARINCL)
+#if	defined(HASWIDECHAR)
+# if	defined(WIDECHARINCL)
 #include WIDECHARINCL
-#endif	/* defined(HASWIDECHAR) && defined(WIDECHARINCL) */
+# endif	/* defined(WIDECHARINCL) */
+# if	defined(HASWCTYPE_H)
+#include <wctype.h>
+# endif	/* defined(HASWCTYPE_H) */
+#endif	/* defined(HASWIDECHAR) */
 
 
 /*
@@ -68,8 +73,6 @@ _PROTOTYPE(static int handleint,(int sig));
 _PROTOTYPE(static void handleint,(int sig));
 #endif	/* defined(HASINTSIGNAL) */
 
-_PROTOTYPE(static char *safepup,(unsigned int c, int *cl));
-
 
 /*
  * Local variables
@@ -133,7 +136,7 @@ build_Nl(d)
 void
 childx()
 {
-	int at, sx;
+	static int at, sx;
 	pid_t wpid;
 
 	if (Cpid > 1) {
@@ -224,9 +227,9 @@ compdev(a1, a2)
 	    return(-1);
 	if ((dev_t)((*p1)->rdev) > (dev_t)((*p2)->rdev))
 	    return(1);
-	if ((ino_t)((*p1)->inode) < (ino_t)((*p2)->inode))
+	if ((INODETYPE)((*p1)->inode) < (INODETYPE)((*p2)->inode))
 	    return(-1);
-	if ((ino_t)((*p1)->inode) > (ino_t)((*p2)->inode))
+	if ((INODETYPE)((*p1)->inode) > (INODETYPE)((*p2)->inode))
 	    return(1);
 	return(strcmp((*p1)->name, (*p2)->name));
 }
@@ -289,14 +292,14 @@ doinchild(fn, fp, rbuf, rbln)
 		 * Begin the child process.
 		 */
 
-		    int fd, nd, r_al, r_rbln;
-		    char r_arg[MAXPATHLEN], r_rbuf[MAXPATHLEN];
+		    int fd, r_al, r_rbln;
+		    char r_arg[MAXPATHLEN+1], r_rbuf[MAXPATHLEN+1];
 		    int (*r_fn)();
 		/*
-		 * Close all open file descriptors except Pipes[0] and
+		 * Close sufficient open file descriptors except Pipes[0] and
 		 * Pipes[3].
 		 */
-		    for (fd = 0, nd = GET_MAX_FD(); fd < nd; fd++) {
+		    for (fd = 0; fd < MaxFd; fd++) {
 			if (fd == Pipes[0] || fd == Pipes[3])
 			    continue;
 			(void) close(fd);
@@ -337,7 +340,7 @@ doinchild(fn, fp, rbuf, rbln)
 			||  write(Pipes[3], r_rbuf, r_rbln) != r_rbln)
 			    break;
 		    }
-		    _exit(0);
+		    (void) _exit(0);
 		}
 	    /*
 	     * Continue in the parent process to finish the setup.
@@ -486,6 +489,228 @@ enter_dev_ch(m)
 }
 
 
+/*
+ * enter_IPstate() -- enter a TCP or UDP state
+ */
+
+void
+enter_IPstate(ty, nm, nr)
+	char *ty;			/* type -- TCP or UDP */
+	char *nm;			/* state name (may be NULL) */
+	int nr;				/* state number */
+{
+
+#if	defined(USE_LIB_PRINT_TCPTPI)
+	TcpNstates = nr;
+#else	/* !defined(USE_LIB_PRINT_TCPTPI) */
+
+	int al, i, j, oc, nn, ns, off, tx;
+	char *cp;
+	MALLOC_S len;
+/*
+ * Check the type name and set the type index.
+ */
+	if (!ty) {
+	    (void) fprintf(stderr,
+		"%s: no type specified to enter_IPstate()\n", Pn);
+	    Exit(1);
+	}
+	if (!strcmp(ty, "TCP"))
+	    tx = 0;
+	else if (!strcmp(ty, "UDP"))
+	    tx = 1;
+	else {
+	    (void) fprintf(stderr, "%s: unknown type for enter_IPstate: %s\n",
+		Pn, ty);
+	    Exit(1);
+	}
+/*
+ * If the name argument is NULL, reduce the allocated table to its minimum
+ * size.
+ */
+	if (!nm) {
+	    if (tx) {
+		if (UdpSt) {
+		    if (!UdpNstates) {
+			(void) free((MALLOC_P *)UdpSt);
+			UdpSt = (char **)NULL;
+		    }
+		    if (UdpNstates < UdpStAlloc) {
+			len = (MALLOC_S)(UdpNstates * sizeof(char *));
+			if (!(UdpSt = (char **)realloc((MALLOC_P *)UdpSt, len)))
+			{
+			    (void) fprintf(stderr,
+				"%s: can't reduce UdpSt[]\n", Pn);
+			    Exit(1);
+			}
+		    }
+		    UdpStAlloc = UdpNstates;
+		}
+	    } else {
+		if (TcpSt) {
+		    if (!TcpNstates) {
+			(void) free((MALLOC_P *)TcpSt);
+			TcpSt = (char **)NULL;
+		    }
+		    if (TcpNstates < TcpStAlloc) {
+			len = (MALLOC_S)(TcpNstates * sizeof(char *));
+			if (!(TcpSt = (char **)realloc((MALLOC_P *)TcpSt, len)))
+			{
+			    (void) fprintf(stderr,
+				"%s: can't reduce TcpSt[]\n", Pn);
+			    Exit(1);
+			}
+		    }
+		    TcpStAlloc = TcpNstates;
+		}
+	    }
+	    return;
+	}
+/*
+ * Check the name and number.
+ */
+	if ((len = (size_t)strlen(nm)) < 1) {
+	    (void) fprintf(stderr,
+		"%s: bad %s name (\"%s\"), number=%d\n", Pn, ty, nm, nr);
+	    Exit(1);
+	}
+/*
+ * Make a copy of the name.
+ */
+	if (!(cp = mkstrcpy(nm, (MALLOC_S *)NULL))) {
+	    (void) fprintf(stderr,
+		"%s: enter_IPstate(): no %s space for %s\n",
+		Pn, ty, nm);
+	    Exit(1);
+	}
+/*
+ * Set the necessary offset for using nr as an index.  If it is
+ * a new offset, adjust previous entries.
+ */
+	if ((nr < 0) && ((off = -nr) > (tx ? UdpStOff : TcpStOff))) {
+	    if (tx ? UdpSt : TcpSt) {
+
+	    /*
+	     * A new, larger offset (smaller negative state number) could mean
+	     * a previously allocated state table must be enlarged and its
+	     * previous entries moved.
+	     */
+		oc = off - (tx ? UdpStOff : TcpStOff);
+		al = tx ? UdpStAlloc : TcpStAlloc;
+		ns = tx ? UdpNstates : TcpNstates;
+		if ((nn = ns + oc) >= al) {
+		    while ((nn + 5) > al) {
+			al += TCPUDPALLOC;
+		    }
+		    len = (MALLOC_S)(al * sizeof(char *));
+		    if (tx) {
+			if (!(UdpSt = (char **)realloc((MALLOC_P *)UdpSt, len)))
+			    goto no_IP_space;
+			UdpStAlloc = al;
+		    } else {
+			if (!(TcpSt = (char **)realloc((MALLOC_P *)TcpSt, len)))
+			    goto no_IP_space;
+			TcpStAlloc = al;
+		    }
+		    for (i = 0, j = oc; i < oc; i++, j++) {
+			if (tx) {
+			    if (i < UdpNstates)
+				UdpSt[j] = UdpSt[i];
+			    UdpSt[i] = (char *)NULL;
+			} else {
+			    if (i < TcpNstates)
+				TcpSt[j] = TcpSt[i];
+			    TcpSt[i] = (char *)NULL;
+			}
+		    }
+		    if (tx)
+			UdpNstates += oc;
+		    else
+			TcpNstates += oc;
+		}
+	    }
+	    if (tx)
+		UdpStOff = off;
+	    else
+		TcpStOff = off;
+	}
+/*
+ * Enter name as {Tc|Ud}pSt[nr + {Tc|Ud}pStOff].
+ *
+ * Allocate space, as required.
+ */
+	al = tx ? UdpStAlloc : TcpStAlloc;
+	off = tx ? UdpStOff : TcpStOff;
+	nn = nr + off + 1;
+	if (nn > al) {
+	    i = tx ? UdpNstates : TcpNstates;
+	    while ((nn + 5) > al) {
+		al += TCPUDPALLOC;
+	    }
+	    len = (MALLOC_S)(al * sizeof(char *));
+	    if (tx) {
+		if (UdpSt)
+		    UdpSt = (char **)realloc((MALLOC_P *)UdpSt, len);
+		else
+		    UdpSt = (char **)malloc(len);
+		if (!UdpSt) {
+
+no_IP_space:
+
+		    (void) fprintf(stderr, "%s: no %s state space\n", Pn, ty);
+		    Exit(1);
+		}
+		UdpNstates = nn;
+		UdpStAlloc = al;
+	    } else {
+		if (TcpSt)
+		    TcpSt = (char **)realloc((MALLOC_P *)TcpSt, len);
+		else
+		    TcpSt = (char **)malloc(len);
+		if (!TcpSt)
+		    goto no_IP_space;
+		TcpNstates = nn;
+		TcpStAlloc = al;
+	    }
+	    while (i < al) {
+		if (tx)
+		    UdpSt[i] = (char *)NULL;
+		else
+		    TcpSt[i] = (char *)NULL;
+		i++;
+	    }
+	} else {
+	    if (tx) {
+		if (nn > UdpNstates)
+		    UdpNstates = nn;
+	    } else {
+		if (nn > TcpNstates)
+		    TcpNstates = nn;
+	    }
+	}
+	if (tx) {
+	    if (UdpSt[nr + UdpStOff]) {
+
+dup_IP_state:
+
+		(void) fprintf(stderr,
+		    "%s: duplicate %s state %d (already %s): %s\n",
+		    Pn, ty, nr,
+		    tx ? UdpSt[nr + UdpStOff] : TcpSt[nr + TcpStOff],
+		    nm);
+	 	Exit(1);
+	    }
+	    UdpSt[nr + UdpStOff] = cp;
+	} else {
+	    if (TcpSt[nr + TcpStOff])
+		goto dup_IP_state;
+	    TcpSt[nr + TcpStOff] = cp;
+	}
+#endif	/* defined(USE_LIB_PRINT_TCPTPI) */
+
+}
+
+
 /*
  * enter_nm() - enter name in local file structure
  */
@@ -800,18 +1025,18 @@ char *
 Readlink(arg)
 	char *arg;			/* argument to be interpreted */
 {
-	char abuf[MAXPATHLEN];
+	char abuf[MAXPATHLEN+1];
 	int alen;
 	char *ap;
 	char *argp1, *argp2;
 	int i, len, llen, slen;
-	char lbuf[MAXPATHLEN];
+	char lbuf[MAXPATHLEN+1];
 	static char *op = (char *)NULL;
 	static int ss = 0;
 	char *s1;
 	static char **stk = (char **)NULL;
 	static int sx = 0;
-	char tbuf[MAXPATHLEN];
+	char tbuf[MAXPATHLEN+1];
 /*
  * See if avoiding kernel blocks.
  */
@@ -835,7 +1060,7 @@ Readlink(arg)
 	for (alen = 0, ap = abuf, argp1 = argp2 = arg; *argp2; argp1 = argp2 ) {
 	    for (argp2 = argp1 + 1; *argp2 && *argp2 != '/'; argp2++)
 		;
-	    if ((len = argp2 - arg) >= MAXPATHLEN) {
+	    if ((len = argp2 - arg) >= (int)sizeof(tbuf)) {
 
 path_too_long:
 		if (!Fwarn) {
@@ -964,9 +1189,10 @@ no_readlink_space:
 	}
 	if (++sx > ss) {
 	    if (!stk)
-		stk = (char **)malloc(sizeof(char *) * sx);
+		stk = (char **)malloc((MALLOC_S)(sizeof(char *) * sx));
 	    else
-		stk = (char **)realloc(stk, sizeof(char *) * sx);
+		stk = (char **)realloc((MALLOC_P *)stk,
+					(MALLOC_S)(sizeof(char *) * sx));
 	    if (!stk)
 		goto no_readlink_space;
 	    ss = sx;
@@ -1086,10 +1312,10 @@ readstqinit(addr, buf)
  *	   cl = strlen(printable equivalent)
  */
 
-static char *
+char *
 safepup(c, cl)
 	unsigned int c;			/* unprintable (i.e., !isprint())
-					 * character */
+					 * character  and '\\' */
 	int *cl;			/* returned printable strlen -- NULL if
 					 * no return needed */
 {
@@ -1122,6 +1348,9 @@ safepup(c, cl)
 	} else if (c == 0xff) {
 	    rp = "^?";
 	    len = 2;
+	} else if (c == '\\') {
+	    rp = "\\\\";
+	    len = 2;
 	} else {
 	    (void) snpf(up, sizeof(up), "\\x%02x", (int)(c & 0xff));
 	    rp = up;
@@ -1154,8 +1383,11 @@ safestrlen(sp, flags)
 	c = (flags & 2) ? ' ' : '\0';
 	if (sp) {
 	    for (; *sp; sp++) {
-		if (!isprint((unsigned char)*sp) || *sp == c) {
-		    if (*sp < 0x20 || (unsigned char)*sp == 0xff)
+		if (!isprint((unsigned char)*sp)
+		||  (*sp == '\\') || (*sp == c))
+		{
+		    if ((*sp < 0x20) || ((unsigned char)*sp == 0xff)
+		    ||  (*sp == '\\'))
 			len += 2;		/* length of \. or ^. form */
 		    else
 			len += 4;		/* length of "\x%02x" printf */
@@ -1191,7 +1423,7 @@ safestrprt(sp, fs, flags)
 					 */
 {
 	char c;
-	int ip, lnc, lnt, sl;
+	int lnc, lnt, sl;
 
 #if	defined(HASWIDECHAR)
 	wchar_t w;
@@ -1229,7 +1461,7 @@ safestrprt(sp, fs, flags)
 		lnc = 1;
 #endif	/* defined(HASWIDECHAR) */
 
-		if (isprint((unsigned char)*sp) && *sp != c)
+		if ((*sp != '\\') && isprint((unsigned char)*sp) && *sp != c)
 		    putc((int)(*sp & 0xff), fs);
 		else {
 		    if ((flags & 8) && (*sp == '\n') && !*(sp + 1))
@@ -1278,7 +1510,7 @@ safestrprtn(sp, len, fs, flags)
 	if (sp) {
 	    c = (flags & 2) ? ' ' : '\0';
 	    for (i = 0; i < len && *sp; sp++) {
-		if (isprint((unsigned char)*sp) && *sp != c) {
+		if ((*sp != '\\') && isprint((unsigned char)*sp) && *sp != c) {
 		    putc((int)(*sp & 0xff), fs);
 		    i++;
 		} else {
@@ -1370,7 +1602,7 @@ x2dev(s, d)
 	char *s;			/* ASCII string */
 	dev_t *d;			/* device receptacle */
 {
-	char c, *cp, *cp1;
+	char *cp, *cp1;
 	int n;
 	dev_t r;
 
